﻿using System;
using System.Linq;
using Buddy.BehaviorTree;
using Buddy.Common;
using Buddy.CommonBot;
using Buddy.Swtor;
using DefaultCombat.Dynamics;
using Action = Buddy.BehaviorTree.Action;

namespace DefaultCombat.Routines
{
    // Darkness Assassin v1.0 - Cystacae/Edited by Neo93
    // Known Bugs:  Can't get behind on pull, Also same as Madness Assassin that randomly Force Lightning may interfere.  I only say may because I have yet to see it but Madness code is relatively the same.
    public static class AssassinDarkness
    {

        [Behavior(BehaviorType.Pull)]
        [Class(CharacterClass.Inquisitor, AdvancedClass.Assassin, SkillTreeId.AssassinDarkness)]
        public static Composite AssassinDarknessPull()
        {
            return new PrioritySelector(
                Spell.Cast("Force Speed", ret => BuddyTor.Me.CurrentTarget.Distance >= Global.MeleeDist && BuddyTor.Me.CurrentTarget.Distance <= 4f),
                Movement.MoveTo(ret => BuddyTor.Me.CurrentTarget.Position, Global.MeleeDist),
                Spell.Cast("Maul", ret => BuddyTor.Me.HasBuff("Stealth"))
            );
        }

        [Behavior(BehaviorType.Combat)]
        [Class(CharacterClass.Inquisitor, AdvancedClass.Assassin, SkillTreeId.AssassinDarkness)]
        public static Composite AssassinDarknessCombat()
        {
            return new PrioritySelector(
                Spell.WaitForCast(),
                Movement.StopInRange(Global.MeleeDist),

                //***Generel***
                Spell.Cast("Force Speed", ret => BuddyTor.Me.CurrentTarget.Distance >= Global.MeleeDist && BuddyTor.Me.CurrentTarget.Distance <= 4f),
                Spell.Cast("Surging Charge", ret => !BuddyTor.Me.HasBuff("Surging Charge")),
                Spell.Cast("Unbreakable Will", ret => BuddyTor.Me.IsStunned),
                Spell.Cast("Recklessness", castWhen => BuddyTor.Me.InCombat && !BuddyTor.Me.HasBuff("Recklessness")),
                Spell.Cast("Dark Ward", castWhen => BuddyTor.Me.InCombat && !BuddyTor.Me.HasBuff("Dark Ward")),
                Spell.Cast("Dark Charge", castWhen => BuddyTor.Me.InCombat && !BuddyTor.Me.HasBuff("Dark Charge")),

                //**Defensive**
                Spell.Cast("Deflection", ret => BuddyTor.Me.InCombat && BuddyTor.Me.HealthPercent <= 40),
                Spell.Cast("Force Shroud", ret => BuddyTor.Me.InCombat && BuddyTor.Me.HealthPercent <= 20),

                //**CC**
                Spell.Cast("Whirlwind", on => Helpers.ccTarget(), castWhen => Helpers.Targets.Count() >= 4),

                //*Interrupts*
                Spell.Cast("Jolt", ret => BuddyTor.Me.CurrentTarget.IsCasting),
                Spell.Cast("Electrocute", ret => BuddyTor.Me.CurrentTarget.IsCasting),
                Spell.Cast("Low Slash", ret => BuddyTor.Me.CurrentTarget.IsCasting),
                Spell.Cast("Overload", ret => BuddyTor.Me.CurrentTarget.IsCasting),


                //Rotation
                Spell.Cast("Maul", castWhen => BuddyTor.Me.HasBuff("Stealth")),
                Spell.Cast("Force Lightning", castWhen => BuddyTor.Me.CurrentTarget.HealthPercent >= 50 && !AbilityManager.HasAbility("Harnessed Darkness")),
                Spell.Cast("Force Lightning", ret => BuddyTor.Me.HasBuff("Harnessed Darkness") && BuddyTor.Me.Buffs.FirstOrDefault(B => B.Name == "Harnessed Darkness").Stacks > 2),
                Spell.Cast("Wither", on => Helpers.ccTarget(), castWhen => Helpers.Targets.Count() >= 2),
                Spell.Cast("Discharge", on => Helpers.ccTarget(), castWhen => Helpers.Targets.Count() >= 2),
                Spell.Cast("Shock", ret => BuddyTor.Me.HasBuff("Energize")),
                Spell.Cast("Crushing Darkness"),
                Spell.Cast("Assassinate", castWhen => BuddyTor.Me.CurrentTarget.HealthPercent <= 30),				
                Spell.Cast("Thrash"),
                Spell.Cast("Saber Strike"),
				

                //Movement
                Movement.MoveTo(ret => BuddyTor.Me.CurrentTarget.Position, Global.MeleeDist)
                );
        }

        [Behavior(BehaviorType.OutOfCombat)]
        [Class(CharacterClass.Inquisitor, AdvancedClass.Assassin, SkillTreeId.AssassinDarkness)]
        public static Composite AssassinDarknessOutOfCombat()
        {
            return new PrioritySelector(
                Spell.Cast("Stealth", castWhen => !BuddyTor.Me.HasBuff("Stealth")),
                Spell.Cast("Surging Charge", ret => !BuddyTor.Me.HasBuff("Surging Charge"))
            );
        }
    }
}