﻿using System;
using System.Threading;
using System.Linq;
using System.Windows;
using System.Windows.Forms;

using Buddy.Common;
using Buddy.BehaviorTree;
using Buddy.CommonBot;
using Buddy.Swtor;
using Buddy.Swtor.Objects;
using Buddy.Common.Math;
using Buddy.Navigation;
using DefaultCombat.Dynamics;

using Action = Buddy.BehaviorTree.Action;

using System.Collections.Generic;
using System.Diagnostics;
using System.ComponentModel;

/* Notes:
    1.  DefaultCombat.cs has issues in that it will 'wait for cast' on Rest spells - so - scan for your rest spell name, and either just add an "xx" at the beginning of it (so it will fail as unknown spell) or remove the 
 *      string from the list it's in altogether (list normally used in routine _ooc).  Rests are handled in OOC() anyhow, so their def in DefaultCombat.cs is moot anyhow.
 *  2.  If having issues with your Spec not being recognized:  (a) First verify that you've loaded your Skill Tree (k key to view/update in SWTOR) and if still an issue, (b) Update SpecHandler.cs to force a return of 
 *      the appropriate tree (first function in SpecHandler.cs) - ex: "return tree3;" .  Make a copy of the original line and comment-it-out first so you have something to go back to just-in-case.
 *  3.  DoRandomGrind flag - if you set this to true, the bot will run around and kill non-friendlies (scanning for LOS aggros first, then non-LOS aggros, and then anything nearby that isn't friendly). As noted below, 
 *      ONLY RUN THIS WITH THE COMBATBOT (CombatBot.xml) PROFILE.  This code will do all of the running around, harvesting, etc.
 *  4.  This code (MercHelpers.cs) handles:
 *      - Harvesting corpses with appropriate skills (Bio and Scav) - it will auto-detect skills.  If you have an issue with skill levels on corpse-harvesting, just move to a lower-level area until you're up-to-par.
 *      - Out-Of-Combat (OOC):  Will rest until you're good and then start moving again, and WILL NOT rest for the full duration of the spell (eliminates wasted resting)
 *      - Companion Harvests:  Will wait for the comp to finish the harvest - default behavior would often 'mount-up' before the companion had even finished their harvest.
 *      - Combat and Casting have been completely overridden - I've had major issues with almost every class I've played - and thus created this.
 *          - Higher-Level - PvP  I avoid the use of PrioritySelectors etc as much as possible - in PvP with all of the errors that'd come up, it was useless originally.  Far-From-Perfect now in PvP, but ALOT better.
 *            IF YOU'RE RUNNING THIS IN PVP - RUN WITH THE COMBATBOT (CombatBot.xml) - otherwise attempted-path-generation - you're killing the path server, and wasting alot of your own CPU, much less messing
 *            everything up in general.  In PvP, you'll need to do the moving on your own, these scripts won't do that - they leave that up to you.  Meshing for battlegrounds is more than a little sketchy at this point.
 *          - Move if necessary/in-range to the current target
 *          - If In-Group, DO NOT AGGRO MOBS NOT ALREADY IN COMBAT (this was a major PITA with the original Default code in FPs, Ops, etc)
 *          - Check for a dead/null target - if comp has target and it's not friendly and not dead, target that.  Default for me would often sit there, with an invalid target, doing nothing while my comp was 
 *            getting waxxed.  If comp has no target or an error comes back, just grab the nearest mob.
 *          - Check for Debuffs:  If any found, cast the cleansing spell.
 *          - Check for low health, heal if necessary
 *          - Check Companion Health, heal if necessary
 *          - If In-Group - Check for nearby players, heal if necessary - shielding and healing the lowest-health first.
 *          - If target is close, cast close-combat/push-back spells
 *          - High-Damage spells
 *          - Lower-Damage spells
 *          - Catch-All no-energy spell (white damage)
 *  5.  So far as the default scripts for the current class - rename ALL of the existing class scripts (for your class) with a .csx extension prior to copying these in.  Otherwise there may be a risk of the wrong 
 *      routines getting assigned on bot startup (especially with the SpecHandler bug noted above - if it isn't handled).
 *  
 * The above can really be taken and applied to any class - just an update of spell names and removal of the 'group heal' calls as necessary.
*
 * Sleeps you see throughout the code - GrayMagic DOES have some lag - give it time to retrieve the value - and for the game to process any incoming updates, commands, etc.  In short, Sleeps are done due to GrayMagic
 * instability - you don't want GrayMagic 'falling into an error vortex' you (or it) can't get out of.
 * 
 * Issues I've Seen and are Unresolved (for Now) - Mainly with BuddyWing and not these scripts.  BuddyWing is AMAZING in all it can do - but it's not perfect:
 *  1.  Periodically, movement doesn't happen - even though the calls are definitely being made, no movement occurs.  Not often, but happens.
 *  2.  At a Higher-Level within the .EXE - when inventory runs low - and it starts trying to run to a Vendor to clean up inv, memory-consumption goes buck-wild (not an issue with these scripts, in the .EXE itself).
 *  3.  Questing Scripts - issues in alot of places - requires alot of babysitting (more-or-less depending on whether you're Emp or Repub, script/level/etc)
 *      - In questing scripts, tags:
 *          - UseObject - generates periodic 'frame-locks' - so you'll get a 'frame-by-frame' effect, slow and annoying.
 *          - UseItem - if trying to use it on a mob especially in combat, useless - tries to use it on the wrong mob, or not-at-all
 *          
 * General Setup Notes for BuddyWing if you're having issues:
 *   1. BuddyWing .exe needs to run on it's own core, and run with a High priority.
 *   2. SWTOR.exe Processes need to run on an AboveNormal priority, on another core (reduces frame-locking lag, I won't go into that). Launcher.exe kicks off SWTOR.exe, so the call to Launcher.exe is the key.
 *   3. In order to make things easier with (1) and (2) I'd recommend you download the SysInternals tools (google 'win7 sysinternals'), set up a .bat that calls psexec for the appropriate Affinity and Priority on each - 
 *      you should be able to google all the help you need for both options.
 *   (2) Is done in order to improve the response in SWTOR so even if you're doing other things like browsing, spreadsheets, etc - it will ensure that SWTOR doesn't lag like hell in the background.
 *   4. UseLockboxes.cs - Plug-In:  If you're having issues with crashes every time you stop - rename it to UseLockboxes.csx.  It will no longer run, but with the above and this rename - should make things more stable.
 *                        (routines in that code that don't seem to process all-that-well when BuddyWing issues a Stop event).  
 *                        
 *      Ditto on AutoEquip.  Fewer Plug-Ins = Fewer Stop-Crashes.
 *      
 *   Layout:
 *     Vars
 *     XXCombat() routines
 *     OOCXX() routines
 *     WaitForHealsXX() routines
 */

public static class DoIt
{
    public static void StopIt()
    {
        if (BuddyTor.Me.InCombat) return;

        Logging.Write("FightAtJoes: RESTARTING THE BOT.");
        BotMain.CurrentBot.Stop();
    }

    public static void StartIt()
    {
        Logging.Write("FightAtJoes: Sleeping for 10 seconds after Stop() just to be safe...");
        //Thread.Sleep(10000);
        while (BotMain.IsRunning) Thread.Sleep(10000);
        //Thread.Sleep(10000);
        //Buddy.CommonBot.RoutineManager.Reload();
        Thread.Sleep(10000);
        BotMain.Start();
        BotMain.CurrentBot.Start();
    }

    public static void PadFunction()
    {
        Debug.Write("blah");
    }

}

//namespace Buddy.Swtor
//{
//    class Mounts
//    {
//        private static string OverrideMount = "";
//        public static string GetFastestAvailableMount()
//        {
//            if (OverrideMount != "") return OverrideMount;
//            return "";
//        }
//    }

//}

namespace DefaultCombat
{
    public static class MercHelpers
    {
        // Combat Vars set up in order to eliminate repeated polling of many affected address(es)/values

        private static int BotSleepFreq = 240;                                          // Bot restart frequency - Seconds Count Check during OOC() - Leave less than 120 for NONE

        public static bool PvP = false;                                                      // PvP Flag - certain var loads skipped with this - far from perfect even with tweaks I've done, but 'mostly' functional

        private static bool DoRandomGrind = false;                                         // ONLY RUN THIS WITH CombatBot.xml - Park character in a Mob-Rich Area first - It doesn't need any waypoints - it will find/target and kill the closest nonfriendly, and repeat.
        private static CombatToughness DRGMinToughness = CombatToughness.Standard;        // Min Combat Toughness with DoRandomGrind - if doing this above CombatToughness.Standard, part it in a spot where you KNOW there will be alot of these.

        private static bool ScanForNodes = true;                                          // Set to true for Node-Scanning w/DoRandomGrind - If you're bent on levelling, set this to false - you won't hit nodes, but it will run faster.
        private static int MaxNodesBetweenFights = 2;                                     // Max nodes to harvest between fights, IF ScanForNodes is true

        private static bool HarvestCorpses = true;                                          // Harvest corpses if you have de skills on given corpse (i.e. strong Droid=Scav, etc)
        
        private static bool AlwaysTargetWeakest = true;                                  // If true, DO NOT use the default-targeting mechanism - go for the weakest surrounding, can slow things down slightly.

        private static System.Windows.Forms.Keys MovementToggleHotKey = Keys.Delete;                // Key to toggle Movement on/off for FPs etc - Credit on this (code immediately using these 'Keys Variables') goes to ChinaJade and Wingit.
        private static System.Windows.Forms.Keys TargWeakestToggleHotKey = Keys.Home;       // Key to toggle Movement on/off for FPs etc - Credit on this (code immediately using these 'Keys Variables') goes to ChinaJade and Wingit.
        private static System.Windows.Forms.Keys ForceCombatHotKey = Keys.NumPad0;
//        private static System.Windows.Forms.Keys ForceCombatHotKey = Keys.NumPad0;      
        private static bool KillFest = false;                                              // With DRG:  If not at least 10 lootables around, just KILL - more XP/hr.
                                                                                          // If false, VERIFY THAT YOU HAVE 'Auto-Target Closest Enemy' set up in your SWTOR Preferences (Control 'selection')
        private static float PullDist= 1.50f;                                               // Min value of 15 here - override the Pull Distance (to alleviate the 'Kill Everything in Sight' Syndrome)

        private static Buddy.CommonBot.Settings.GlobalSettings GS = new Buddy.CommonBot.Settings.GlobalSettings();

        private static bool PullIG = false;                                               // ONLY set this to true if you want to pull while In-Group for whatever reason
        private static bool UseMarksSurvRot = false;                                       // Go with the (Sniper ONLY) Marksmanship 'Survival Rotation' (by Xanathos) vs. mine (mine is geared for a healer companion, his is not)

        private static string MountName = Buddy.Swtor.Mounts.GetFastestAvailableMount();    // For use with DoRandomGrind (since the OOC pretty much overrides any 'idle time' for normal profile operations)

        private const float MaxDist = 50;                                                   // 10x factor internally in SWTOR - For use with DoRandomGrind

        private static bool InitsComplete = false;
        private static bool HasScav = false;
        private static bool HasBio = false;
        private static bool HasArch = false;
        private static bool HasSlicing = false;
        private static bool IsRanged = false;
        private static string MyBlacklist = "";

        private static bool UseAOE = true;

        private static DateTime datLLSl = DateTime.Now;                                      // Last Long Sleep - to try and help even further with GrayMagic errors
        private static DateTime datLC;                                                      // Last check for harvestables
        private static DateTime datLCs;                                                      // Last cast - Used for check in ForcedCast - mainly PvP
        private static DateTime datLCL;                                                     // Last Check Local (used locally in various routines)
        private static DateTime datLFP;                                                     // Last Forced Pulse() - ONLY do this once every 5 seconds max
        private static DateTime datLCC = DateTime.Now;                                        // Last Comp Check
        private static DateTime datLGC = DateTime.Now;                                      // Last GC (runs roughly once-per-minute Out-Of-Combat)
        private static DateTime datLLS = DateTime.Now;                                      // Last Loot Scan:  To Be Used w/KillFest (only scan for loot every 2 minutes)
        private static DateTime datLMPU = DateTime.Now;                                     // Last Medpack Used - 2-minute restriction
        private static DateTime datLBC = DateTime.Now;                                      // Last Buff Check - Only check buffs every 20s
        private static DateTime CS;                                                         // Last GC
        private static ulong CG;                                                            // Cur Target GUI(ID)

        private static bool MovementAllowed = true;
        private static IMovementProvider DefaultMP = Navigator.MovementProvider;
        private static INavigationProvider DefaultNP = Navigator.NavigationProvider;
        private static IMovementProvider CurrentMP = Navigator.MovementProvider;
        private static INavigationProvider CurrentNP = Navigator.NavigationProvider;
        private static NoMovement_MovementProvider NoMovementProvider = new NoMovement_MovementProvider();
        private static NoMovement_NavigationProvider NoNavigationProvider = new NoMovement_NavigationProvider();

        //private static LootTargeting
        private static TorPlayer LHUnit = null;                                             // Low-Health Unit
        private static TorCharacter NGUnit = null;                                             // 'Needs Guarding' Unit - for Tanking
        private static TorNpc NAUnit = null;                                                // 'Needs Aggrod' NPC Unit - for Tanking
        private static TorCharacter CT = null;                                              // Curr Targ
        private static TorCharacter CompTarg = null;                                        // Comp Targ
        private static TorCharacter LastComp = null;
        //private static TorCharacter Comp = null;

        private static string MyBuffs = "";                                                 // Buffs/Affects List (String, Semicolon-Delimited)
        private static string CTDebuffs = "";                                               // Cur Targ Debuffs List (String, Semicolon-Delimited)

        private static Vector3 TP;                                                          // Targ Pos
        private static SkillTreeId MySpec = SpecHandler.GetSpec();
        private static float HP = 0;                                                        // Health Pct
        private static float EP = 0;                                                        // Energy Pct
        private static float TD = 0;                                                        // Targ Dist
        private static bool TC = false;                                                     // Targ Casting
        private static float RS = 0;                                                        // Resource Stat
        private static float THP = 0;                                                       // Targ Health Pct
        public static bool IG = false;                                                      // In-Group - Important Re:  Healing and Pulls
        private static float CHP = 0;                                                       // Comp Health Pct
        private static CombatToughness CTT;                                                 // Cur Targ Toughness
        private static int TWR30 = 0;                                                       // Target Count within 30m
        private static int TWR10 = 0;                                                       // Targets within 10m
        private static int TWR5 = 0;                                                        // Targets within 5m
        private static float LHH = 0;                                                       // Lowest-Health Health Pct (for In-Group Healing)
        private static float LHD = 0;                                                       // Lowest-Health Dist
        private static Vector3 LHP;                                                         // Lowest-Health Pct

        //private static TorAbility taStealth = null;
        //private static TorAbility taUppercut = null;
        //private static TorAbility taJab = null;
        //private static TorAbility taBash = null;
        //private static TorAbility taUnity = null;

        private static EffectResult er;

        //public static IEnumerable<TorCharacter> allUnits;
        private static Buddy.CommonBot.IBot IB;

        private static Thread thStart;
        private static Thread thStop;

        public static BotEvent BotEvent_OnBotStart = new BotEvent(OnBotStart);
        public static BotEvent BotEvent_OnBotStop = new BotEvent(OnBotStop);

        private static void DoRestart()
        {

            if (Me.InCombat || Me.CurrentTarget !=null) return;

            Logging.Write("FightAtJoes:  Stopping and restarting the bot.");
            Thread.Sleep(1000);
            Buddy.Navigation.Navigator.Clear();
            Thread.Sleep(1000);
            Buddy.CommonBot.Poi.Clear("Fight at Joe's:  Stopping the Bot.");
            Thread.Sleep(1000);

            datLLSl = DateTime.Now.AddSeconds(13);

            thStart = null;
            thStop = null;

            //return;

            thStart = new System.Threading.Thread(DoIt.StartIt);
            thStop = new System.Threading.Thread(DoIt.StopIt);

            thStart.Start();
            thStop.Start();

        }

        private static void OnBotStart(IBot bot)
        {
            //try { datLFP = DateTime.Now; Buddy.CommonBot.BotMain.CurrentBot.Pulse(); }
            //catch { }
            InitsComplete = false;
            datLLSl = DateTime.Now;
            OOC();
        }

        private static void OnBotStop(IBot bot)
        {
        }

        private static void CheckGC()
        {
            if (DateTime.Now.Subtract(datLGC).TotalSeconds >= 120)
            {
                System.GC.Collect();
                System.GC.GetTotalMemory(true);
                datLGC = DateTime.Now;
                Thread.Sleep(1000);
            }

            if (!IG && BotSleepFreq >= 120 && DateTime.Now.Subtract(datLLSl).TotalSeconds >= BotSleepFreq)
            {
                //DoRestart();
                if (Me.InCombat || Me.CurrentTarget != null) return;
                Logging.Write("FightAtJoes:  Sleeping for 10 seconds.  Attempt to try and help with the GrayMagic issues - this rest will occur Out-of-Combat once every " + (BotSleepFreq/60).ToString("0.0") + " minutes.");

                // 'Object Reset Query' JIC Everything has Gone Haywire - ALL-INCLUSIVE
                float d = 0;
                try { foreach (TorObject to in ObjectManager.GetObjects<TorObject>()) { d += to.Distance; } }
                catch { }

                datLLSl = DateTime.Now.AddSeconds(13);
                Thread.Sleep(10000);
            }

        }

        private static void MoveIfNec()
        {
            MoveIfNec(2.8f);
        }

        private static void MoveIfNec(float dist)
        {
            // Doesn't always move - so the 2nd block is used Just-In-Case

            if (CT == null) return;
            if (!PvP) if (CT.IsDead) return;
            if (CTT == CombatToughness.Player) return;

            MoveTo(CT, dist);

        }

        private static void WaitForCast()
        {
            DateTime dStart = DateTime.Now;
            while (BuddyTor.Me.IsCasting && DateTime.Now.Subtract(dStart).TotalSeconds < 4) Thread.Sleep(500);
        }

        private static void StopMoving()
        {
            if (!MovementAllowed) return;
            // Loop is due to issues with stopping - often doesn't stop, so keep trying until it does
            Buddy.Swtor.Input.MoveStopAll();
            Buddy.Swtor.Movement.Stop(MovementDirection.Forward);
            Thread.Sleep(50);
            //while (BuddyTor.Me.IsMoving)
            //{
            //    // Double-Condom approach - tried many, many, ways of stopping, this is the ONLY RELIABLE way of doing it I've found to-date
            //    Buddy.Swtor.Movement.Stop(MovementDirection.Forward);
            //    //Buddy.Swtor.Input.MoveStopAll();
            //    //Buddy.CommonBot.CommonBehaviors.MoveStop();
            //    Thread.Sleep(400);

            //}
            //Buddy.Swtor.Movement.Stop(MovementDirection.Forward);
        }

        private static void MoveTo(TorCharacter theUnit, float dist)
        {
            MoveTo(theUnit, dist, null);
        }

        private static void MoveTo(TorCharacter theUnit, float dist, string LeapAbil)
        {
            StopMoving();           // To (try and) prevent 'The Chicken Dance'
            datLCL = DateTime.Now;

            if (theUnit == null || dist <= 0) return;

            if (theUnit.Distance > dist || !theUnit.InLineOfSight)
            {
                //theUnit.Face();
                //Thread.Sleep(100);
                //theUnit.Face();
                //Thread.Sleep(100);

                // (Try to) Force the movement start - otherwise - may just sit there... doing nothing.
                // I've done the best I could with this routine - Face() doesn't work 1/2 the time, and the movement gets just a little fucked-up here and there.
                //Buddy.Swtor.Movement.Move(Buddy.Swtor.MovementDirection.Forward, System.TimeSpan.FromMilliseconds(100));
                //Thread.Sleep(100);

                StopMoving();
                while (theUnit != null && DateTime.Now.Subtract(datLCL).TotalSeconds <= 20 && (theUnit.Distance > dist || !theUnit.InLineOfSight))
                {
                    MercCast(LeapAbil, CT);
                    if (CT.Distance > dist || !theUnit.InLineOfSight)
                    {
                        Buddy.Navigation.Navigator.MoveTo(theUnit.Position);
                        Thread.Sleep(100);
                    }
                }
                StopMoving();
            }
        }

        private static bool MeOrCompInCombat()
        {
            if (Me.InCombat) return true;
            if (BuddyTor.Me.Companion != null) if (BuddyTor.Me.Companion.InCombat) return true;
            return false;
        }

        private static bool UpdateTarget()
        {
            if (PvP) { CT = (TorCharacter)(ObjectManager.GetObjects<TorPlayer>().OrderBy(t => t.HealthPercent).Where(d => d.Distance <= 2.80f && !d.IsFriendly)).FirstOrDefault(); TD = CT.Distance; return true; }

            if (Comp != null) if (CT == Comp) { CT = null; BuddyTor.Me.ClearTarget(); }

            if (!AlwaysTargetWeakest || !Me.InCombat) { CT = Me.CurrentTarget; return false; }
            if (Me.CurrentTarget != null) if (!IsRanged && !Me.CurrentTarget.IsDead) {CT = Me.CurrentTarget; return false;}
            TorCharacter CTPrev = CT;
            try
            {
                CT = Helpers.Targets.OrderBy(t => t.Health).Where(t => t.HealthPercent > 0 && !t.IsStunned && !t.IsFrozen).FirstOrDefault();
                if (CT == null) CT = Me.CurrentTarget; 
            }
            catch { CT = Me.CurrentTarget; }
            if (CT == null) CT = Me.CurrentTarget;
            if (CT != null) TD = CT.Distance;
            return CT != CTPrev;
        }

        // For use mainly in groups, trying to determine if the current target's target is a player, and it IS in combat
        public static bool TargetInCombat()
        {
            if (CT == null) return false;
            if (PvP) return true;
            if (CTT == CombatToughness.Player) if (!CT.IsFriendly) return true;  // PvP - you target an enemy player, go-to-town

            if (IG && !PullIG)
            {
                if (!CT.InCombat) return false;    // If we're in Ops/FP/etc - DON'T ATTACK SHIT THAT HASN'T BEEN PULLED
                if (CT.IsStunned) { BuddyTor.Me.ClearTarget(); Thread.Sleep(300); CT = null; return false; }
                if (CT.CurrentTarget == null) return false; else if (CT.CurrentTarget.Toughness != CombatToughness.Player) return false;
            }
            return true;
        }

        public static void UseMedPackInInventory()
        {
            if (DateTime.Now.Subtract(datLMPU).TotalSeconds < 120) return;

            foreach (var o in BuddyTor.Me.InventoryEquipment)
            {
                if (o.Name.Contains("Medpac"))
                    //if (o.IsUsable) 
                    { o.Use(); o.Interact(); datLMPU = DateTime.Now; return; }
            }
        }

        public static bool HaveBuffStacks(string strBuff, int intCount)
        {
            try { if (MyBuffs.Contains(strBuff)) foreach (TorEffect te in Me.Buffs) if (te.Name.Contains(strBuff)) if (te.Stacks >= intCount) return true; }
            catch { }
            return false;
        }

        public static bool HasDebuffStacks(string strDebuff, int intCount)
        {
            try { if (CTDebuffs.Contains(strDebuff)) foreach (TorEffect te in CT.Debuffs) if (te.Name.Contains(strDebuff)) if (te.Stacks >= intCount) return true; }
            catch { }
            return false;
        }

        public static void FirebugCombat()
        {

            datLCL = DateTime.Now;

            Thread.Sleep(100);
            LoadVars();

            if (IG && !CT.InCombat) return;    // If we're in Ops/FP/etc - DON'T ATTACK SHIT THAT HASN'T BEEN PULLED
            //if (UpdateTarget()) return;

            MoveIfNec();
            if (!IG && BuddyTor.Me.IsMoving) StopMoving();

            Thread.Sleep(100);

            if (MercCast("Determination", BuddyTor.Me.IsStunned)) return;

            if (MercCast("Healing Scan", HP <= 70f)) return;
            if (MercCast("Rapid Scan", HP <= 70f)) return;
            if (MercCast("Kolto Overload", HP <= 70f)) return;
            if (MercCast("Energy Shield", HP <= 60f)) return;

            HandleDebuffs();

            if (MercCast("Disabling Shot", TC)) return;

            if (MercCast("Hunter's Boon", !MyBuffs.Contains("Hunter's Boon"))) return;
            if (MercCast("Heroic Moment: On the Trail", HP <= 50)) return;
            if (MercCast("Power Surge")) return;

            if (MercCast("Electro Dart", TD <= 1.0f)) return;
            if (MercCast("Jet Boost", TD <= .8f)) return;
            if (MercCast("Shoulder Slam", TD <= .8f)) return;
            if (MercCast("Rocket Punch", TD <= .8f)) return;
            //MercCast("Flame Thrower", TD >= .01f && TD <= .8f);

            if (MercCast("Thermal Sensor Override", RS >= 23f)) return;
            if (MercCast("Vent Heat", RS >= 80f)) return;
            if (Comp != null) if (MercCast("Chaff Flare", HP <= 70f)) return;
            if (MercCast("Supercharged Gas", !MyBuffs.Contains("Supercharged Gas"))) return;

            if (MercCast("Incendiary Missile", CT, !CTDebuffs.Contains("Burning (Tech)"))) return;
            if (MercCast("Thermal Detonator")) return;

            if (UseAOE)
            {
                if (MercCast("Explosive Dart")) return;

                if (MercCastOnGround("Death from Above", TD >= .6f && BuddyTor.Me.ResourceStat < 70, CT)) return;

                if (MercCast("Fusion Missile", StrongOrGreater(CT))) return;
                if (MercCast("Rail Shot")) return;
                if (CTT == CombatToughness.Player) MercCastOnGround("Sweeping Blasters", TD >= .1f && RS < 65f && (TWR30 > 2 || StrongOrGreater(CT)), CT);
                if (MercCastOnGround("Sweeping Blasters", TD >= .1f && RS < 65 && (TWR30 > 2 || StrongOrGreater(CT)), CT)) return;
                if (MercCast("Missile Blast", TD <= .8f && !StrongOrGreater(CT))) return;
            }

            if (MercCast("Rapid Shots", HP < 70 && RS > 70f)) return;

            if (MercCast("Unload")) return;
            if (MercCast("Power Shot", MyBuffs.Contains("Power Surge"))) return;

            if (MercCast("Rapid Shots")) return;

            Buddy.CommonBot.AbilityManager.Cast("Rapid Shots", CT);   // Forced Cast - CanCast gets just a little flaky
        }


        public static void BGCombat()
        {
            if (Me.IsDead || Me.HealthPercent <= .50f) return;

            //Thread.Sleep(500);
            try
            {
                datLCL = DateTime.Now;

                WaitForCast();

                Thread.Sleep(300);
                LoadVars();

                if (CT == null) { Logging.Write("No target.  Exiting."); return; }

                if (MercCast("Kolto Shell", BuddyTor.Me, !MyBuffs.Contains("Kolto Shell") && HP <= 85)) return;
                if (MercCastOnGround("Kolto Missile", HP < 60, Me)) return;
                if (MercCast("Rapid Scan", HP <= 70)) return;
                if (MercCast("Kolto Overload", HP <= 70)) return;
                if (MercCast("Energy Shield", HP <= 60)) return;

                if (BGHeals(85f)) return;
                if (BGCheckCompHealth()) return;

                if (!IG) MoveIfNec();
                if (!IG && BuddyTor.Me.IsMoving) StopMoving();

                Thread.Sleep(100);

                if (MercCast("Emergency Scan", HP < 70)) return;
                if (MercCast("Determination", BuddyTor.Me.IsStunned)) return;

                if (!TargetInCombat()) return;

                if (MercCast("Power Shot", MyBuffs.Contains("Power Surge"))) return;
                if (MercCast("Disabling Shot", TC)) return;
                if (MercCast("Electro Dart", TC)) return;

                if (MercCast("Hunter's Boon", !MyBuffs.Contains("Hunter's Boon"))) return;
                if (MercCast("Heroic Moment: On the Trail", HP <= 50)) return;
                if (MercCast("Power Surge")) return;

                if (MercCast("Punch", TD <= .40f)) return;
                if (MercCast("Jab", TD <= .40f)) return;
                if (MercCast("Uppercut", TD <= .40f)) return;

                if (MercCast("Jet Boost", TD <= .8f)) return;
                if (MercCast("Shoulder Slam", TD <= .8f)) return;
                if (MercCast("Rocket Punch", TD <= .8f)) return;
                //MercCast("Flame Thrower", TD >= .01f && TD <= .8f);

                if (MercCast("Thermal Sensor Override", RS >= 23)) return;
                if (MercCast("Vent Heat", RS >= 80)) return;
                if (Comp != null) if (MercCast("Chaff Flare", HP <= 70)) return;
                if (MercCast("Supercharged Gas", !MyBuffs.Contains("Supercharged Gas"))) return;

                if (!IG || (IG && RS <= 40))
                {
                    if (MercCast("Rapid Shots", HP < 70 && RS > 70)) return;

                    if (MercCast("Unload")) return;
                    if (MercCast("Rail Shot")) return;

                    if (UseAOE)
                    {
                        if (MercCast("Fusion Missile", StrongOrGreater(CT))) return;
                        if (MercCast("Explosive Dart")) return;
                        if (MercCastOnGround("Death from Above", TD >= .6f && BuddyTor.Me.ResourceStat < 70, CT)) return;
                        if (CTT == CombatToughness.Player) MercCastOnGround("Sweeping Blasters", TD >= .1f && RS < 65 && (TWR30 > 2 || StrongOrGreater(CT)), CT);
                        if (MercCastOnGround("Sweeping Blasters", TD >= .1f && RS < 65 && (TWR30 > 2 || StrongOrGreater(CT)), CT)) return;
                        if (MercCast("Missile Blast", TD <= .8f && !StrongOrGreater(CT))) return;
                    }

                }

                //Buddy.Navigation.Navigator.MoveTo(BuddyTor.Me.CurrentTarget.Position);
                if (MercCast("Rapid Shots")) return;

                Buddy.CommonBot.AbilityManager.Cast("Rapid Shots", CT);   // Forced Cast - CanCast gets just a little flaky
            }
            catch { Buddy.CommonBot.AbilityManager.Cast("Rapid Shots", Me.CurrentTarget); }
        }

        public static void ArsenalCombat()
        {
            if (Me.IsDead || Me.HealthPercent <= .50f) return;

            //Thread.Sleep(500);
            try
            {
                datLCL = DateTime.Now;

                WaitForCast();

                Thread.Sleep(300);
                LoadVars();

                if (CT == null) { Logging.Write("No target.  Exiting."); return; }

                if (MercCast("Rapid Scan", HP <= 70)) return;
                if (MercCast("Healing Scan", HP <= 70)) return;
                if (MercCast("Kolto Overload", HP <= 70)) return;
                if (MercCast("Energy Shield", HP <= 60)) return;

//                if (BGHeals(85f)) return;
                if (BGCheckCompHealth()) return;

                if (!IG) MoveIfNec();
                if (!IG && BuddyTor.Me.IsMoving) StopMoving();

                Thread.Sleep(100);

                if (MercCast("High Velocity Gas Cylinder", !MyBuffs.Contains("High Velocity Gas Cylinder"))) return;
                if (MercCast("Emergency Scan", HP < 70)) return;
                if (MercCast("Determination", BuddyTor.Me.IsStunned)) return;

                if (!TargetInCombat()) return;

                if (MercCast("Power Shot", MyBuffs.Contains("Power Surge"))) return;
                if (MercCast("Rail Shot", HaveBuffStacks("Tracer Lock", 4))) return;

                if (MercCast("Disabling Shot", TC)) return;
                if (MercCast("Electro Dart", TC)) return;

                if (MercCast("Hunter's Boon", !MyBuffs.Contains("Hunter's Boon"))) return;
                if (MercCast("Heroic Moment: On the Trail", HP <= 50)) return;
                if (MercCast("Power Surge")) return;

                if (MercCast("Jet Boost", TD <= .8f)) return;
                if (MercCast("Shoulder Slam", TD <= .8f)) return;
                if (MercCast("Rocket Punch", TD <= .8f)) return;
                //MercCast("Flame Thrower", TD >= .01f && TD <= .8f);

                if (MercCast("Thermal Sensor Override", RS >= 23)) return;
                if (MercCast("Vent Heat", RS >= 80)) return;
                if (Comp != null) if (MercCast("Chaff Flare", HP <= 70)) return;
                if (MercCast("Supercharged Gas", !MyBuffs.Contains("Supercharged Gas"))) return;

                //if (!IG || (IG && RS <= 40))
                {
                    if (MercCast("Rapid Shots", HP < 70 && RS > 70)) return;

                    if (MercCastOnGround("Death from Above", TD >= .6f && BuddyTor.Me.ResourceStat < 70, CT)) return;

                    if (MercCast("Tracer Missile", StrongOrGreater(CT) && !HasDebuffStacks("Heat Signature", 4))) return;
                    if (MercCast("Heatseeker Missiles", StrongOrGreater(CT))) return;
                    if (MercCast("Fusion Missile", StrongOrGreater(CT))) return;

                    if (MercCast("Explosive Dart")) return;
                    if (MercCast("Unload", MyBuffs.Contains("Barrage"))) return;
                    if (MercCast("Rail Shot")) return;

                    if (CTT == CombatToughness.Player) MercCastOnGround("Sweeping Blasters", TD >= .1f && RS < 65 && (TWR30 > 2 || StrongOrGreater(CT)), CT);
                    if (MercCastOnGround("Sweeping Blasters", TD >= .1f && RS < 65 && (TWR30 > 2 || StrongOrGreater(CT)), CT)) return;
                    if (MercCast("Missile Blast", TD <= .8f && !StrongOrGreater(CT))) return;

                }

                //Buddy.Navigation.Navigator.MoveTo(BuddyTor.Me.CurrentTarget.Position);
                if (MercCast("Rapid Shots")) return;

                //Buddy.CommonBot.AbilityManager.Cast("Rapid Shots", CT);   // Forced Cast - CanCast gets just a little flaky
            }
            catch { 
                //Buddy.CommonBot.AbilityManager.Cast("Rapid Shots", Me.CurrentTarget); 
            }
        }

        public static void CombatMedicCombat()
        {

            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(300);
            LoadVars();

            if (CT == null) { Logging.Write("No target.  Exiting."); return; }

            if (!IG) if (CTT >= CombatToughness.Boss1) MoveIfNec(9.90f); else MoveIfNec();

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();

            //Thread.Sleep(100);
            if (MercCast("Tenacity", BuddyTor.Me.IsStunned)) return;

            HandleDebuffs();

            if (MercCast("Recharge Cells", RS <= 6f)) return;
            if (MercCast("Supercharge Cells")) return;

            if (MercCast("Bacta Infusion", Me, HP < 70)) return;
            if (MercCast("Advanced Medical Probe", Me, HP < 70)) return;
            if (MercCast("Adrenaline Rush", Me, HP < 70)) return;
            if (MercCastOnGround("Kolto Bomb", HP <= 70, Me)) return;
            if (MercCast("Medical Probe", Me, HP < 70)) return;

            if (MercCast("Reactive Shield", HP <= 50f)) return;
            if (MercCast("Hammer Shot", HP < 50 && RS < 3f)) return;

            if (MercCast("Concussive Round", HP < 50)) { BuddyTor.Me.ClearTarget(); return; }

            if (CMCheckCompHealth()) return;
            if (CMHeals(80)) return;

            if (IG && !TargetInCombat()) { Logging.Write("In-Group and Target not in combat.... exiting combat."); return; }    // If we're in Ops/FP/etc - DON'T ATTACK SHIT THAT HASN'T BEEN PULLED

            if (MercCast("Trauma Probe", Me, !MyBuffs.Contains("Trauma Probe"))) return;
            if (MercCast("Cryo Grenade", TC || TD <= 1.0f)) return;
            if (MercCast("Disabling Shot", TC)) return;

            if(!IG || (IG && RS > 4f)) {
                if (MercCast("Blitz", TD <= .50f)) return;
                if (MercCast("Stockstrike", TD <= .40f)) return;
                if (MercFaceAndCast("Pulse Cannon", TD <= 1.0f)) return;
                if (MercCast("Concussion Charge", TD <= .50f)) return;
                if (MercCast("Explosive Round", TD <= .40f && StrongOrGreater(CT))) return;

                if (MercCast("Tech Override", StrongOrGreater(CT))) return;
                if (MercCast("Charged Bolts", MyBuffs.Contains("Tech Override"))) return;
                if (MercCast("Plasma Grenade", StrongOrGreater(CT))) return;
                if (MercCastOnGround("Mortar Volley", TD >= .6f && StrongOrGreater(CT), CT, true)) return;
                if (MercCast("High Impact Bolt")) return;
                if (MercCast("Sticky Grenade")) return;
                if (MercCast("Full Auto")) return;

                if (MercCastOnGround("Hail of Bolts", BuddyTor.Me.ResourceStat > 6f, CT, true)) return;
                if (MercCast("Hammer Shot")) return;
            }
            if (MercCast("Hammer Shot")) return;

            //Buddy.CommonBot.AbilityManager.Cast("Rapid Shots", CT);   // Forced Cast - CanCast gets just a little flaky
        }

        public static void SSCombat()
        {

            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(300);
            LoadVars();

            if (CT == null) { Logging.Write("No target.  Exiting."); return; }

            if (!IG) if (CTT >= CombatToughness.Boss1) MoveIfNec(9.90f); else MoveIfNec();

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();

            if (!CT.InCombat) MercCast("Harpoon");

            //Thread.Sleep(100);
            if (MercCast("Tenacity", BuddyTor.Me.IsStunned)) return;
            if (MercCast("Reactive Shield", HP <= 50f)) return;
            if (MercCast("Adrenaline Rush", HP <= 50f)) return;

            if (MercCast("Recharge Cells", RS <= 6f)) return;
            if (MercCast("Reserve Powercell", RS <= 6f)) return;
            if (MercCast("Supercharge Cells")) return;

            if (IG && !TargetInCombat()) return;

            if (MercCast("Energy Blast", RS <= 8f)) return;
            if (MercCast("Neural Jolt")) return;
            if (MercCast("Stockstrike", TD <= .40f)) return;

            if (MercCast("Explosive Surge", TWR5 >= 2)) return;

            if (MercCastOnGround("Mortar Volley", TD >= .6f && (StrongOrGreater(CT) || TWR30 > 2), CT, true)) return;

            if (MercCast("High Impact Bolt")) return;
            if (MercCast("Pulse Cannon")) return;
            if (MercCast("Sticky Grenade")) return;
            if (MercCast("Ion Pulse", TD <= 1.0f)) return;

            if (MercCast("Battle Focus")) return;

            if (MercCast("Full Auto")) return;

            if (MercCast("Hammer Shot", HP < 50 && RS < 3f)) return;
            if (MercCast("Concussive Round", HP < 50)) { BuddyTor.Me.ClearTarget(); return; }

            if (MercCast("Cryo Grenade", TC || TD <= 1.0f)) return;
            if (MercCast("Disabling Shot", TC)) return;

            if (!IG || (IG && RS > 4f))
            {
                if (MercCast("Blitz", TD <= .50f)) return;
                if (MercFaceAndCast("Pulse Cannon", TD <= 1.0f)) return;
                if (MercCast("Concussion Charge", TD <= .50f)) return;
                if (MercCast("Explosive Round", TD <= .40f && !StrongOrGreater(CT))) return;

                if (MercCast("Tech Override", StrongOrGreater(CT))) return;
                if (MercCast("Charged Bolts", MyBuffs.Contains("Tech Override"))) return;
                if (MercCast("Plasma Grenade", StrongOrGreater(CT))) return;
                if (MercCast("Sticky Grenade")) return;

                if (MercCastOnGround("Hail of Bolts", BuddyTor.Me.ResourceStat > 6f, CT, true)) return;
                if (MercCast("Hammer Shot")) return;
            }
            if (MercCast("Hammer Shot")) return;

            //Buddy.CommonBot.AbilityManager.Cast("Rapid Shots", CT);   // Forced Cast - CanCast gets just a little flaky
        }

        public static bool HandleGuardingKinetic()
        {

            //if (!IG) return false;

            CheckForGuarding();

            if (NAUnit != null) try
                {
                    CT = NAUnit;

                    if (MercCast("Force Pull", NAUnit)) return true;
                    if (MercCast("Mind Control", NAUnit)) return true;
                    if (MercCast("Mass Mind Control", NAUnit, NAUnit.Distance <= 1.50f)) return true;
                    if (MercCast("Guard", NGUnit)) return true;

                }
                catch { }
            return false;
        }

        public static void ConsularKineticCombat()
        {
            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(200);

            LoadVars();

            //if (!IG) if (CTT >= CombatToughness.Boss1) MoveIfNec(9.90f); else MoveIfNec();

            MercBuffSelf("Combat Technique", !(MyBuffs.Contains("Combat Technique") || Me.InCombat));

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();
            if (CT == null && Me.CurrentTarget != null) CT = Me.CurrentTarget;

            if (CT == null) { Logging.Write("No target.  Exiting."); return; }

            if (!CT.InCombat && CT.HealthPercent > .2f)
            {
                if (CT.Distance >= 1.0f && CT.Distance <= 3.0f) if (MercCast("Force Pull")) return;
                if (StrongOrGreater(CT)) if (MercCast("Mind Crush")) return;

                MercCast("Stealth");
                while (CT.Distance > .15f) MoveTo(CT, .15f);
                if (MercCast("Shadow Strike", CT.Distance <= .20f)) if (MercCast("Double Strike")) return;
            }

            MercCast("Force of Will", Me.IsStunned);
            if (IG) if (HandleGuardingKinetic()) return;

            //MoveTo(CT, 1.00f, "Force Speed");
            if (MercCast("Project", RS > 45f && TD >= .6f)) return;
            if (MercCast("Telekinetic Throw", TD >= .6f)) return;

            if (TD > 1.5f) if (MercCast("Force Pull")) return;
            MoveTo(CT, .490f, "Force Speed");

            if (MercCast("Mind Snap", TC)) return;
            if (MercCast("Mind Control", TC && StrongOrGreater(CT))) return;
            //if (MercCast("Force Wave", TD <= .50f && TC && StrongOrGreater(CT))) return;
            if (MercCast("Force Wave", TWR10 >= 3)) return;
            if (MercCast("Force Stun", StrongOrGreater(CT) && CTT <= CombatToughness.Boss1)) return;

            //**Defensive**
            if (HP <= 30) UseMedPackInInventory();

            if (MercCast("Force Wave", TWR10 >= 2)) return;

            if (MercCast("Heroic Moment: Channel the Force", HP <= 40f)) return;
            if (MercCast("Heroic Moment:  Channel the Force", HP <= 40f)) return;

            if (MercCast("Kinetic Ward", HP < 80f)) return;
            if (MercCast("Battle Readiness", HP <= 50f)) return;
            if (MercCast("Force Potency", HP <= 50f)) return;
            if (MercCast("Resilience", HP <= 50f)) return;
            if (MercCast("Deflection", HP <= 50f)) return;
            
            if (MercCast("Force Potency", HP <= 70f || (CTT >= CombatToughness.Boss1) || (TWR10 >= 3))) return;

            //if (TD <= .45f)
            //{
                //if (taJab.IsValid) if (BuddyTor.Me.AbilityActivate(taJab, CT, out er)) return;
                //if (BuddyTor.Me.AbilityActivate(taBash, CT, out er)) return;
                //if (BuddyTor.Me.AbilityActivate(taUppercut, CT, out er)) return;
            //}

            //if (HP < 30) if (BuddyTor.Me.AbilityActivate(taUnity, Me, out er))return;

            if (MercCast("Force Breach", (TD <= 1.0f && StrongOrGreater(CT)) || TWR10 >= 3) ) return;
            if (MercCast("Force Wave", TWR10 >= 3)) return;

            if (MercCast("Tumult", TD <= .50f)) return;
            if (MercCast("Spinning Strike", THP <= 30f)) return;

            if (MercCast("Mind Crush", StrongOrGreater(CT) && !CTDebuffs.Contains("Mind Crush") && THP >= 50)) return;
            
            if (MercCast("Project", TD <= 1.0f && RS > 45f)) return;
            if (MercCast("Telekinetic Throw", TD <= 1.0f)) return;
            
            if (MercCast("Force Slow", StrongOrGreater(CT))) return;

            //Rotation
            if(MercCast("Slow Time", !CTDebuffs.Contains("Slow Time (Force)"))) return;
			if(MercCast("Force Breach", TWR10 >= 3)) return;
            if (MercCast("Whirling Blow", TWR10 >= 3)) return;
            if (MercCast("Project", RS > 45f)) return;
            if (MercCast("Telekinetic Throw")) return;				

            if(MercCast("Double Strike", RS > 65f)) return;
                
            MercCast("Saber Strike");

        }

        public static void SniperMarksmanshipPull()
        {
            CT = Me.CurrentTarget;
            if (!TargetInCombat()) return;

            LoadVars();
            MoveTo(CT, 2.8f);

            //Me.CurrentTarget.Face();
            //Thread.Sleep(600);
            MercCast("Crouch");
            Thread.Sleep(600);
            MercCast("Ambush");
            return;
        }

        // Ops Medicine BLOWS, weak, WEAK, heals
        public static void MedicineCombat()
        {
            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(300);

            LoadVars();

            MercBuffSelf("Coordination", true);
            MoveTo(CT, 2.80f);

            //if (!IG) if (CTT >= CombatToughness.Boss1) MoveIfNec(9.90f); else MoveIfNec();

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();
            if (CT == null && Me.CurrentTarget != null) CT = Me.CurrentTarget;

            if (CT == null) { Logging.Write("No target.  Exiting."); return; }
            if (CT == Comp) { BuddyTor.Me.ClearTarget(); return; }

            if (!TargetInCombat()) return;

            if (MercCast("Escape", Me.IsStunned)) return;

            MercCast("Crouch", Me, !Me.IsCoverAffected);
            if (!CT.InCombat)
            {
                //EffectResult er;
                //foreach (TorAbility ta in ((TorPlayer)Comp).KnownAbilitiesContainer) if (ta.Name.Contains("Attack")) { if (ta.IsValid) ((TorPlayer)Comp).AbilityActivate(ta,CT,out er); break; }
                //MercCast("Attack");
                Thread.Sleep(600);
                //MercCast("Attack");
                MercCast("Snipe");
            }

            if (MercCast("Distraction", (IG && HP <= 50) || (StrongOrGreater(CT) && TC))) return;
            if (MercCast("Flash Bang", TD <= .50f)) return;
            if (MercCast("Carbine Burst", MyBuffs.Contains("Tactical Advantage") && StrongOrGreater(CT) && TD <= .50f)) return;
            if (MercCast("Debilitate", StrongOrGreater(CT) && TC)) return;
            if (MercCast("Fragmentation Grenade", !IG && StrongOrGreater(CT) && TC)) return;

            //if (HP <= 70) MercCast("Rifle Shot");    // Force a Rifle-Shot on healing so we get at least a LITTLE DAMAGE in - may help with a Heals Catch-22

            if (MercCast("Heroic Moment:  Advanced Recon", Me, HP <= 25)) return;
            if (MercCast("Evasion", Me, HP <= 30)) return;
            if (MercCast("Kolto Probe", Me, HP <= 70 && !MyBuffs.Contains("Kolto Probe"))) return;
            if (MercCast("Surgical Probe", HP <= 70 && MyBuffs.Contains("Tactical Advantage"))) return;
            if (MercCast("Kolto Injection", Me, HP <= 60)) return;
            if (MercCast("Kolto Infusion", Me, HP <= 70)) return;
            if (MercCast("Adrenaline Probe", EP <= 20)) return;
            if (MercCast("Stim Boost", EP <= 60 && MyBuffs.Contains("Tactical Advantage"))) return;

            if (IG) if (MedicineHeals(80)) return;

            if (MercCast("Eviscerate", TD <= .50f)) return;
            if (MercCast("Shiv", TD <= .50f)) return;
            if (MercCast("Sever Tendon", TD <= 1.0f)) return;

            if (MedicineCheckCompHealth()) return;

            if (MercCast("Rifle Shot", RS <= 30 && (StrongOrGreater(CT) || IG))) return;

            if (MercCast("Fragmentation Grenade", !IG)) return;
            if (MercCast("Explosive Probe")) return;
            if (MercCast("Corrosive Dart", !CTDebuffs.Contains("Poisoned (Tech)"))) return;
            if (MercCast("Flash Bang")) return;

            if (MercCast("Carbine Burst", TWR10 >= 3)) return;

            if (MercCast("Snipe")) return;
            if (MercCast("Rifle Shot")) return;

        }

        // Ops Medicine BLOWS, weak, WEAK, heals
        public static void LethalityCombat()
        {
            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(100);

            LoadVars();

            if (!TargetInCombat()) return;

            MercBuffSelf("Coordination", true);
            MoveTo(CT, 2.8f);

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();
            if (CT == null && Me.CurrentTarget != null) CT = Me.CurrentTarget;

            if (CT == null) { Logging.Write("No target.  Exiting."); return; }
            if (CT == Comp) { BuddyTor.Me.ClearTarget(); return; }

            if (MercCast("Escape", Me.IsStunned)) return;

            MercCast("Crouch", Me, !Me.IsCoverAffected);
            if (!CT.InCombat)
            {
                Thread.Sleep(600);
                MercCast("Snipe");
            }

            //MoveTo(CT, .50f, "Rifle Shot");

            if (MercCast("Distraction", (IG && HP <= 50) || (StrongOrGreater(CT) && TC))) return;
            if (MercCast("Flash Bang", TD <= .50f)) return;
            if (MercCast("Carbine Burst", MyBuffs.Contains("Tactical Advantage") && StrongOrGreater(CT) && TD <= .50f)) return;
            if (MercCast("Debilitate", StrongOrGreater(CT) && TC)) return;
            if (MercCast("Fragmentation Grenade", StrongOrGreater(CT) && TC)) return;

            //if (HP <= 70) MercCast("Rifle Shot");    // Force a Rifle-Shot on healing so we get at least a LITTLE DAMAGE in - may help with a Heals Catch-22

            if (MercCast("Shield Probe", Me, !MyBuffs.Contains("Shield Probe"))) return;

            if (MercCast("Heroic Moment:  Advanced Recon", Me, HP <= 25)) return;
            //if (MercCast("Evasion", Me, HP <= 30)) return;
            //if (MercCast("Kolto Probe", Me, HP <= 70 && !MyBuffs.Contains("Kolto Probe"))) return;
            //if (MercCast("Surgical Probe", HP <= 70 && MyBuffs.Contains("Tactical Advantage"))) return;
            if (MercCast("Kolto Injection", Me, HP <= 60)) return;
            if (MercCast("Kolto Infusion", Me, HP <= 70)) return;
            if (MercCast("Adrenaline Probe", EP <= 20)) return;
            if (MercCast("Stim Boost", EP <= 60 && MyBuffs.Contains("Tactical Advantage"))) return;

            if (MercCast("Cull", MyBuffs.Contains("Tactical Advantage") && TWR10 >= 2)) return;

            if (MercCast("Lacerate", TD <= .50f)) return;
            if (MercCast("Eviscerate", TD <= .50f)) return;
            if (MercCast("Shiv", TD <= .50f)) return;
            if (MercCast("Sever Tendon", TD <= 1.0f)) return;

            if (MedicineCheckCompHealth()) return;

            if (MercCast("Rifle Shot", RS <= 30 && (StrongOrGreater(CT) || IG))) return;

            if (MercCast("Debilitate", StrongOrGreater(CT))) return;
            if (MercCast("Headshot", CT.IsStunned)) return;
            if (MercCast("Corrosive Grenade")) return;
            if (MercCast("Fragmentation Grenade", !IG)) return;
            if (MercCast("Explosive Probe")) return;
            if (MercCast("Weakening Blast")) return;
            if (MercCast("Corrosive Dart", !CTDebuffs.Contains("Poisoned (Tech)"))) return;
            if (MercCast("Flash Bang")) return;

            if (MercCast("Carbine Burst", TWR10 >= 3)) return;

            if (MercCast("Snipe")) return;
            if (MercCast("Rifle Shot")) return;

        }

        // Ops Medicine BLOWS, weak, WEAK, heals
        public static void ConcealmentCombat()
        {
            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(100);

            LoadVars();

            if (!TargetInCombat()) return;

            MercBuffSelf("Coordination", true);
            MoveTo(CT, 2.8f);

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();
            if (CT == null && Me.CurrentTarget != null) CT = Me.CurrentTarget;

            if (CT == null) { Logging.Write("No target.  Exiting."); return; }
            if (CT == Comp) { BuddyTor.Me.ClearTarget(); return; }

            if (!CT.InCombat && CT.HealthPercent > .2f)
            {
                MercCast("Stealth");
                while (CT.Distance > .15f) MoveTo(CT, .15f);
                MercCast("Acid Blade");
                MercCast("Ambush", CT.Distance <= .20f);
                MercCast("Backstab", CT.Distance <= .20f);
            }

            if (MercCast("Escape", Me.IsStunned)) return;

            MercCast("Crouch", Me, !MyBuffs.Contains("Crouch"));

            MoveTo(CT, .50f, "Rifle Shot");

            if (MercCast("Acid Blade", !CTDebuffs.Contains("Acid Blade"))) return;

            if (MercCast("Followthrough")) return;
            if (MercCast("Takedown")) return;
            if (MercCast("Headshot")) return;
            if (MercCast("Shattershot")) return;

            //if (HP <= 70) MercCast("Rifle Shot");    // Force a Rifle-Shot on healing so we get at least a LITTLE DAMAGE in - may help with a Heals Catch-22

            if (MercCast("Shield Probe", Me, !MyBuffs.Contains("Shield Probe"))) return;

            if (MercCast("Heroic Moment: Advanced Recon", Me, HP <= 25)) return;
            //if (MercCast("Evasion", Me, HP <= 30)) return;
            //if (MercCast("Kolto Probe", Me, HP <= 70 && !MyBuffs.Contains("Kolto Probe"))) return;
            //if (MercCast("Surgical Probe", HP <= 70 && MyBuffs.Contains("Tactical Advantage"))) return;

            if (MercCast("Kolto Injection", Me, HP <= 60)) return;
            if (MercCast("Kolto Infusion", Me, HP <= 70)) return;
            if (MercCast("Adrenaline Probe", EP <= 20)) return;
            if (MercCast("Stim Boost", EP <= 60 && MyBuffs.Contains("Tactical Advantage"))) return;

            if (MercCast("Stim Boost", Me, RS <= 20)) return;
            if (MercCast("Fragmentation Grenade", !IG)) return;

            if (TD <= .50f)
            {
                if (MercCast("Orbital Strike", StrongOrGreater(CT))) return;
                if (MercCast("Shiv")) return;
                if (MercCast("Lacerate", MyBuffs.Contains("Tactical Advantage"))) return;
                if (MercCast("Carbine Burst", MyBuffs.Contains("Tactical Advantage") && StrongOrGreater(CT))) return;
                if (MercCast("Eviscerate")) return;
                if (MercCast("Sever Tendon")) return;
            }

            if (MercCast("Distraction", (IG && HP <= 50) || (StrongOrGreater(CT) && TC))) return;
            if (MercCast("Flash Bang", TD <= .50f)) return;
            if (MercCast("Debilitate", StrongOrGreater(CT) && TC)) return;
            if (MercCast("Fragmentation Grenade", !IG && StrongOrGreater(CT) && TC)) return;

            if (MercCast("Cull", MyBuffs.Contains("Tactical Advantage") && TWR10 >= 2)) return;

            if (MedicineCheckCompHealth()) return;

            if (MercCast("Rifle Shot", RS <= 30 && (StrongOrGreater(CT) || IG))) return;

            if (MercCast("Corrosive Grenade")) return;
            if (MercCast("Debilitate", StrongOrGreater(CT))) return;
            if (MercCast("Headshot", CT.IsStunned)) return;
            if (MercCast("Explosive Probe")) return;
            if (MercCast("Weakening Blast")) return;
            if (MercCast("Corrosive Dart", !CTDebuffs.Contains("Poisoned (Tech)"))) return;
            if (MercCast("Flash Bang")) return;

            if (MercCast("Carbine Burst", TWR10 >= 3)) return;

            if (MercCast("Snipe")) return;
            if (MercCast("Rifle Shot")) return;

        }

        public static void DirtyFightingCombat()
        {
            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(100);

            LoadVars();

            if (!TargetInCombat()) return;

            MercBuffSelf("Lucky Shots", true);
            MoveTo(CT, 2.8f);

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();
            if (CT == null && Me.CurrentTarget != null) CT = Me.CurrentTarget;

            if (CT == null) { Logging.Write("No target.  Exiting."); return; }
            if (CT == Comp) { BuddyTor.Me.ClearTarget(); return; }

            if (MercCast("Escape", Me.IsStunned)) return;

            MercCast("Crouch", Me, !Me.IsCoverAffected);
            if (!CT.InCombat)
            {
                Thread.Sleep(600);
                MercCast("Charged Burst");
            }

            //MoveTo(CT, .50f, "Flurry of Bolts");

            if (MercCast("Distraction", (IG && HP <= 50) || (StrongOrGreater(CT) && TC))) return;
            if (MercCast("Flash Grenade", TD <= .50f)) return;
            if (MercCast("Blaster Volley", MyBuffs.Contains("Upper Hand") && StrongOrGreater(CT) && TD <= .50f)) return;
            if (MercCast("Dirty Kick", StrongOrGreater(CT) && TC)) return;
            if (MercCast("Thermal Grenade", StrongOrGreater(CT) && TC)) return;

            //if (HP <= 70) MercCast("Flurry of Bolts");    // Force a Rifle-Shot on healing so we get at least a LITTLE DAMAGE in - may help with a Heals Catch-22

            if (MercCast("Defense Screen", Me, !MyBuffs.Contains("Defense Screen"))) return;

            if (MercCast("Heroic Moment:  All In", Me, HP <= 25)) return;
            //if (MercCast("Evasion", Me, HP <= 30)) return;
            //if (MercCast("Kolto Probe", Me, HP <= 70 && !MyBuffs.Contains("Kolto Probe"))) return;
            //if (MercCast("Surgical Probe", HP <= 70 && MyBuffs.Contains("Tactical Advantage"))) return;
            if (MercCast("Underworld Medicine", Me, HP <= 60)) return;
            if (MercCast("Kolto Pack", Me, HP <= 70)) return;
            if (MercCast("Cool Head", EP <= 20)) return;
            if (MercCast("Pugnacity", EP <= 60 && MyBuffs.Contains("Upper Hand"))) return;

            if (MercCast("Wounding Shots", MyBuffs.Contains("Upper Hand") && TWR10 >= 2)) return;

            if (MercCast("Whirling Blow", TD <= .50f)) return;
            if (MercCast("Cheap Shot", TD <= .50f)) return;
            if (MercCast("Blaster Whip", TD <= .50f)) return;
            if (MercCast("Tendon Blast", TD <= 1.0f)) return;

            if (MedicineCheckCompHealth()) return;

            if (MercCast("Flurry of Bolts", RS <= 30 && (StrongOrGreater(CT) || IG))) return;

            if (MercCast("Dirty Kick", StrongOrGreater(CT))) return;
            if (MercCast("Cheap Shot", CT.IsStunned)) return;
            if (MercCast("Shrap Bomb")) return;
            if (MercCast("Thermal Grenade", !IG)) return;
            if (MercCast("Sabotage Charge")) return;
            if (MercCast("Hemorrhaging Blast")) return;
            if (MercCast("Vital Shot", !CTDebuffs.Contains("Bleeding (Tech)"))) return;
            if (MercCast("Flash Grenade")) return;

            if (MercCast("Blaster Volley", TWR10 >= 3)) return;

            if (MercCast("Charged Burst")) return;
            if (MercCast("Flurry of Bolts")) return;

        }

        public static void ScrapperCombat()
        {
            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(100);

            LoadVars();

            if (!TargetInCombat()) return;

            MercBuffSelf("Lucky Shots", true);
            MoveTo(CT, 2.8f);

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();
            if (CT == null && Me.CurrentTarget != null) CT = Me.CurrentTarget;

            if (CT == null) { Logging.Write("No target.  Exiting."); return; }
            if (CT == Comp) { BuddyTor.Me.ClearTarget(); return; }

            if (!CT.InCombat && CT.HealthPercent > .2f)
            {
                MercCast("Fletchette Round");
                MercCast("Stealth");
                while (CT.Distance > .15f) MoveTo(CT, .15f);
                MercCast("Fletchette Round");
                MercCast("Shoot First", CT.Distance <= .20f);
            }

            if (MercCast("Escape", Me.IsStunned)) return;

            MercCast("Crouch", Me, !MyBuffs.Contains("Crouch"));

            MoveTo(CT, .50f, "Flurry of Bolts");

            if (MercCast("Fletchette Round", !CTDebuffs.Contains("Fletchette Round"))) return;

            if (MercCast("Quickdraw")) return;
            if (MercCast("Headshot")) return;
            if (MercCast("Flourish Shot")) return;

            //if (HP <= 70) MercCast("Flurry of Bolts");    // Force a Rifle-Shot on healing so we get at least a LITTLE DAMAGE in - may help with a Heals Catch-22

            if (MercCast("Defense Screen", Me, !MyBuffs.Contains("Defense Screen"))) return;

            if (MercCast("Heroic Moment: All In", Me, HP <= 25)) return;
            //if (MercCast("Evasion", Me, HP <= 30)) return;
            //if (MercCast("Kolto Probe", Me, HP <= 70 && !MyBuffs.Contains("Kolto Probe"))) return;
            //if (MercCast("Surgical Probe", HP <= 70 && MyBuffs.Contains("Tactical Advantage"))) return;

            if (MercCast("Underworld Medicine", Me, HP <= 60)) return;
            if (MercCast("Kolto Pack", Me, HP <= 70)) return;
            if (MercCast("Cool Head", EP <= 20)) return;
            if (MercCast("Pugnacity", EP <= 60 && MyBuffs.Contains("Upper Hand"))) return;

            if (MercCast("Pugnacity", Me, RS <= 20)) return;
            if (MercCast("Thermal Grenade", !IG)) return;

            if (TD <= .50f)
            {
                if (MercCast("XS Freighter Flyby", StrongOrGreater(CT))) return;
                if (MercCast("Blaster Whip")) return;
                if (MercCast("Whirling Blow", MyBuffs.Contains("Upper Hand"))) return;
                if (MercCast("Blaster Volley", MyBuffs.Contains("Upper Hand") && StrongOrGreater(CT))) return;
                if (MercCast("Cheap Shot")) return;
                if (MercCast("Tendon Blast")) return;
            }

            if (MercCast("Distraction", (IG && HP <= 50) || (StrongOrGreater(CT) && TC))) return;
            if (MercCast("Flash Grenade", TD <= .50f)) return;
            if (MercCast("Dirty Kick", StrongOrGreater(CT) && TC)) return;
            if (MercCast("Thermal Grenade", !IG && StrongOrGreater(CT) && TC)) return;

            if (MercCast("Wounding Shots", MyBuffs.Contains("Upper Hand") && TWR10 >= 2)) return;

            if (MedicineCheckCompHealth()) return;

            if (MercCast("Flurry of Bolts", RS <= 30 && (StrongOrGreater(CT) || IG))) return;

            if (MercCast("Shrap Bomb")) return;
            if (MercCast("Dirty Kick", StrongOrGreater(CT))) return;
            if (MercCast("Headshot", CT.IsStunned)) return;
            if (MercCast("Sabotage Charge")) return;
            if (MercCast("Hemorrhaging Blast")) return;
            if (MercCast("Vital Shot", !CTDebuffs.Contains("Bleeding (Tech)"))) return;
            if (MercCast("Flash Grenade")) return;

            if (MercCast("Blaster Volley", TWR10 >= 3)) return;

            if (MercCast("Charged Burst")) return;
            if (MercCast("Flurry of Bolts")) return;

        }
        public static void SniperMarksmanshipCombat()
        {
            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(300);

            LoadVars();

            MercBuffSelf("Coordination", true);
            MoveTo(CT, 2.80f);

            //if (!IG) if (CTT >= CombatToughness.Boss1) MoveIfNec(9.90f); else MoveIfNec();

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();
            if (CT == null && Me.CurrentTarget != null) CT = Me.CurrentTarget;

            if (CT == null) { Logging.Write("No target.  Exiting."); return; }

            if (!TargetInCombat()) return;

            MercCast("Crouch", Me, !MyBuffs.Contains("Crouch"));
            if (!CT.InCombat)
            {
                Thread.Sleep(600);
                if (MercCast("Ambush", !CT.InCombat || CT.IsStunned))return;
                if (MercCast("Snipe", !CT.InCombat || CT.IsStunned)) return;
            }

            if (UseMarksSurvRot)
            {

                if (MercCast("Takedown")) return;
                if (MercCast("Headshot")) return;
                if (MercCast("Followthrough")) return;

                if (MercCast("Evasion", HP <= 30f)) return;
                if (MercCast("Shield Probe", HP <= 75f)) return;
                if (MercCast("Countermeasures", MyBuffs.Contains("Shield Probe"))) return;
                if (MercCast("Cover Pulse", TD <= .50f)) return;
                if (MercCast("Target Acquired", StrongOrGreater(CT) && THP >= 50f)) return;
                if (MercCast("Laze Target", StrongOrGreater(CT) && THP >= 50f)) return;
                if (MercCast("Adrenaline Probe", RS <= 50f)) return;
                if (MercCast("Sniper Volley", HP <= 50f || RS <= 40f)) return;

                if (MercCastOnGround("Orbital Strike", CTT >= CombatToughness.Boss1 && THP >= 30f, CT)) return;
                if (MercCast("Fragmentation Grenade", TWR30 >= 3)) return;

                if (MercCast("Distraction", TC && StrongOrGreater(CT))) return;
                if (MercCast("Flash Bang", TWR5 >= 2)) return;
                if (MercCast("Debilitate", TC)) return;

                if (MercCast("Shiv", TD <= .50f)) return;
                if (MercCast("Eviscerate", TD <= .50f)) return;

                if (MercCast("Snipe", MyBuffs.Contains("Snap Shot"))) return;
                if (MercCast("Shatter Shot", !CTDebuffs.Contains("Armor Reduced (Physical)") && StrongOrGreater(CT))) return;
                if (MercCast("Takedown")) return;
                if (MercCast("Ambush", MyBuffs.Contains("Reactive Shot"))) return;
                if (MercCast("Followthrough")) return;
                if (MercCast("Series of Shots")) return;
                if (MercCast("Ambush", MyBuffs.Contains("Followthrough"))) return;
                if (MercCast("Snipe", MyBuffs.Contains("Followthrough"))) return;
                if (MercCast("Snipe", MyBuffs.Contains("Laze Target"))) return;
                if (MercCast("Headshot")) return;
                if (MercCast("Explosive Probe", StrongOrGreater(CT))) return;
                if (MercCast("Corrosive Dart", !CTDebuffs.Contains("Poisoned (Tech)") && StrongOrGreater(CT))) return;
                if (MercCast("Ambush")) return;
                if (MercCast("Snipe")) return;

                if (MercCast("Rifle Shot")) return;
            }
            else
            {

                if (MercCastOnGround("Orbital Strike", CTT >= CombatToughness.Boss1 && THP >= 30f, CT)) return;

                if (MercCast("Followthrough")) return;
                if (MercCast("Headshot", CT.IsStunned)) return;

                if (MercCast("Debilitate", TC)) return;
                if (MercCast("Distraction", TC && StrongOrGreater(CT))) return;

                if (MercCast("Adrenaline Probe", RS <= 20)) return;
                if (MercCast("Sniper Volley", HP <= 50 || RS <= 20)) return;

                if (MercCast("Shield Probe", !MyBuffs.Contains("Shield Probe"))) return;
                if (MercCast("Entrench", HP <= 70f)) return;
                if (MercCast("Ballistic Shield", HP <= 50)) return;
                if (MercCast("Diversion", HP <= 50 && CT.Toughness != CombatToughness.Player)) return;

                    if (HP <= 40f) UseMedPackInInventory();
                    if (MercCast("Fragmentation Grenade")) return;

                if (TD <= .50f)
                {

                    if (MercCast("Cover Pulse")) return;
                    if (MercCast("Flash Bang")) return;
                    if (MercCast("Fragmentation Grenade")) return;
                    if (MercCast("Shiv")) return;
                    if (MercCast("Eviscerate")) return;
                    if (MercCast("Ambush")) return;
                }

                if (MercCast("Explosive Probe")) return;
                if (MercCast("Corrosive Dart", !CTDebuffs.Contains("Poisoned (Tech)"))) return;

                if (MercCast("Headshot")) return;
                if (MercCast("Takedown")) return;
                if (MercCast("Snipe", CTDebuffs.Contains("Laze Target"))) return;
                if (MercCast("Shatter Shot", StrongOrGreater(CT) && !CTDebuffs.Contains("Armor Reduced (Physical)"))) return;
                if (MercCast("Series of Shots")) return;

                if (MercCast("Laze Target", StrongOrGreater(CT) && THP >= 50f)) return;
                if (MercCast("Target Acquired", StrongOrGreater(CT) && THP >= 50f)) return;

                if (MercCast("Debilitate")) return;
                if (MercCast("Leg Shot")) return;

                if (MercCastOnGround("Suppressive Fire", TWR30 >= 2, CT)) return;

                if (MercCast("Ambush")) return;
                if (MercCast("Snipe")) return;

                if (MercCast("Rifle Shot")) return;
            }
            ////Logging.Write(RS.ToString("0.0") + " Forced Cast:  Rifle Shot");
            ////Buddy.CommonBot.AbilityManager.Cast("Rifle Shot", CT);
        }

        public static void MarauderAnnihilationCombat()
        {

            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(100);
            LoadVars();

            if (!TargetInCombat()) return;

            if (MercCast("Juyo Form", !MyBuffs.Contains("Juyo Form"))) return;
            if (MercCast("Unnatural Might", Me, !MyBuffs.Contains("Unnatural Might"))) return;

            if (TD >= .50f && TD <= 3.0f) if (MercCast("Force Charge")) return;

            if (!IG) MoveTo(CT, .490f, "Force Charge");
            //if (!IG) MoveIfNec(.49f);

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();

            //Thread.Sleep(100);
            if (MercCast("Unleash", BuddyTor.Me.IsStunned)) return;
            if (MercCast("Disruption", TC)) return;

            ///HandleDebuffs();

            if (MercCast("Saber Ward", Me, HP <= 50f)) return;
            if (MercCast("Undying Rage", Me, HP <= 20f)) return;
            if (MercCast("Berserk", Me, MyBuffs.Contains("Fury") && HP <= 70)) return;

            if (MercCast("Pommel Strike", CT.IsStunned)) return;

            if (HP <= 70)
            {
                if (MercCast("Frenzy", Me, !MyBuffs.Contains("Fury"))) return;
                if (MercCast("Cloak of Pain", Me)) return;
                if (MercCast("Obfuscate")) return;
                if (MercCast("Intimidating Roar")) return;
                if (MercCast("Deadly Throw")) return;
                if (MercCast("Force Choke")) return;
            }

            if (MercCast("Berserk", Me)) return;
            if (MercCast("Annihilate")) return;
            if (MercCast("Deadly Saber")) return;
            if (MercCast("Force Scream")) return;
            if (MercCast("Rupture")) return;
            if (MercCast("Smash", TWR5 >= 2)) return;
            if (MercCast("Battering Assault")) return;
            if (MercCast("Vicious Throw", THP <= 30f)) return;
            if (MercCast("Ravage")) return;
            if (MercCast("Retaliation")) return;

            if (MercCast("Predation", Me, MyBuffs.Contains("Fury")) && !MyBuffs.Contains("Predation")) return;
            if (MercCast("Bloodthirst", Me, MyBuffs.Contains("Fury") && HP > 50) && !MyBuffs.Contains("Bloodthirst")) return;

            if (MercCast("Vicious Slash")) return;
            if (MercCast("Assault")) return;

            //Buddy.CommonBot.AbilityManager.Cast("Rapid Shots", CT);   // Forced Cast - CanCast gets just a little flaky
        }

        public static void MarauderRageCombat()
        {

            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(100);
            LoadVars();

            if (!TargetInCombat()) return;

            if (MercCast("Shii-Cho Form", !MyBuffs.Contains("Shii-Cho Form"))) return;
            if (MercCast("Unnatural Might", !MyBuffs.Contains("Unnatural Might"))) return;

            if (TD >= .50f && TD <= 3.0f) if (MercCast("Force Charge")) return;

            if (!IG) MoveTo(CT, .490f, "Force Charge");
            //if (!IG) MoveIfNec(.49f);

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();

            //Thread.Sleep(100);
            if (MercCast("Unleash", BuddyTor.Me.IsStunned)) return;
            if (MercCast("Disruption", TC)) return;

            ///HandleDebuffs();

            if (MercCast("Saber Ward", HP <= 50f)) return;
            if (MercCast("Undying Rage", HP <= 20f)) return;
            if (MercCast("Berserk", Me, MyBuffs.Contains("Fury") && HP <= 70)) return;

            if (MercCast("Smash", MyBuffs.Contains("Dominate") || HaveBuffStacks("Shockwave", 4))) return;

            if (MercCast("Frenzy", Me, !MyBuffs.Contains("Fury") && StrongOrGreater(CT))) return;
            if (MercCast("Predation", Me)) return;
            if (MercCast("Bloodthirst", Me, HP > 70)) return;

            if (MercCast("Cloak of Pain", Me)) return;
            if (MercCast("Obfuscate")) return;
            if (MercCast("Force Crush", StrongOrGreater(CT))) return;
            if (MercCast("Rupture")) return;
            if (MercCast("Intimidating Roar", TWR10 >= 2)) return;

            if (MercCast("Force Choke")) return;
            if (MercCast("Obliterate")) return;
            if (MercCast("Vicious Throw", THP <= 30f)) return;
            if (MercCast("Pommel Strike", CT.IsStunned)) return;

            if (MercCast("Battering Assault")) return;

            //if (MercCast("Annihilate")) return;
            if (MercCast("Force Scream")) return;

            if (MercCast("Deadly Throw")) return;

            if (MercCast("Deadly Saber")) return;
            if (MercCast("Predation", MyBuffs.Contains("Fury") && HP < 50) && !MyBuffs.Contains("Predation")) return;
            if (MercCast("Bloodthirst", MyBuffs.Contains("Fury") && HP > 50) && !MyBuffs.Contains("Bloodthirst")) return;

            if (MercCast("Retaliation")) return;
            if (MercCast("Ravage")) return;

            if (MercCast("Deadly Throw")) return;

            if (MercCast("Bloodthirst")) return;

            if (MercCast("Vicious Slash")) return;
            if (MercCast("Assault")) return;

            //Buddy.CommonBot.AbilityManager.Cast("Rapid Shots", CT);   // Forced Cast - CanCast gets just a little flaky
        }

        public static void SeerCombat()
        {

            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(100);
            LoadVars();
            if (!TargetInCombat()) return;

            if (MercCast("Force Valor", !MyBuffs.Contains("Force Valor"))) return;
            if (!IG) if (Me.Level > 10) MoveTo(CT, 2.8f, "Force Speed"); else MoveTo(CT, .50f);
            //if (!IG) MoveIfNec(.49f);

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();

            if (MercCast("Force of Will", Me.IsStunned)) return;
            if (MercCast("Force Potency", Me, HP <= 50f)) return;

            if (MercCast("Force Wave", StrongOrGreater(CT) && TD <= .50f)) return;

            if (MercCast("Force Armor", Me, HP <= 90f && !MyBuffs.Contains("Force Armor"))) return;
            if (MercCast("Force Mend", Me, HP <= 70f)) return;
            if (MercCast("Deliverance", Me, HP <= 70f)) return;
            if (MercCastOnGround("Salvation", HP <= 50f && !MyBuffs.Contains("Salvation"), Me)) return;
            if (MercCast("Benevolence", Me, HP <= 50f)) return;

            if (MercCast("Force Potency", Me.ForcePercent <= 50f)) return;
            if (MercCast("Heroic Moment:  Channel the Force", Me.ForcePercent <= 30f)) return;

            if (IG) if (SeerHeals(80f)) return;
            if (SeerCheckCompHealth()) return;

            if (MercCast("Noble Sacrifice", HP >= 50 && RS <= Me.ForceMax * .2f)) return;

            if (MercCast("Mind Crush", !CT.InCombat)) return;
            if (MercCast("Disturbance", !CT.InCombat)) return;

            if (!IG || Me.ForcePercent > 30f)
            {

                if (MercCast("Mind Snap", TC && StrongOrGreater(CT))) return;
                if (MercCast("Mind Crush", StrongOrGreater(CT))) return;
                if (MercCast("Cloud Mind", StrongOrGreater(CT))) return;
                if (MercCast("Force Stun", StrongOrGreater(CT))) return;
                if (MercCast("Weaken Mind", StrongOrGreater(CT) && !CTDebuffs.Contains("Weaken Mind (Force)"))) return;
                if (MercCast("Force Slow", !CTDebuffs.Contains("Slow (Force)"))) return;

                if (MercCast("Project")) return;
                if (MercCast("Telekinetic Throw")) return;
                if (MercCastOnGround("Forcequake", TWR30 >= 3, CT)) return;
                if (MercCast("Disturbance")) return;
            }

            if (TD <= .50f)
            {
                if (MercCast("Force Wave", TD <= .50f)) return;
                if (MercCast("Tumult")) return;
                if (MercCast("Double Strike")) return;
                if (MercCast("Saber Strike")) return;
                //if (MercCast("Double Strike")) return;
            }

        }

        public static void MadnessCombat()
        {

            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(200);
            LoadVars();
            if (!TargetInCombat()) return;

            if (MercCast("Mark of Power", !MyBuffs.Contains("Mark of Power"))) return;
            if (!IG) MoveTo(CT, 2.8f, "Force Speed");
            //if (!IG) MoveIfNec(.49f);

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();

            if (MercCast("Unbreakable Will", Me.IsStunned)) return;

            if (MercCastOnGround("Death Field", HP <= 50 || StrongOrGreater(CT), CT)) return;

            if (MercCast("Affliction", !CTDebuffs.Contains("Affliction (Force)"))) return;
            if (MercCast("Creeping Terror")) return;

            if (MercCast("Static Barrier", Me, !MyBuffs.Contains("Static Barrier"))) return;
            if (MercCast("Unnatural Preservation", Me, HP <= 70f)) return;
            if (MercCast("Dark Heal", Me, HP <= 70f)) return;
            if (MercCast("Dark Infusion", Me, HP <= 70f)) return;
            if (MercCast("Cloud Mind", Me, CTT != CombatToughness.Player && HP <= 70f)) return;

            if (CorruptionCheckCompHealth()) return;

            if (MercCast("Recklessness", Me.ForcePercent <= 50f)) return;
            if (MercCast("Heroic Moment:  Channel the Force", Me.ForcePercent <= 30f)) return;

            if (MercCast("Consumption", HP > 50 && Me.ForcePercent <= 20)) return;
            //if (SeerHeals(75f)) return;

            if (TD <= .50f)
            {
                if (MercCast("Overload", TD <= .50f)) return;
                if (MercCast("Tumult")) return;
                //if (MercCast("Double Strike")) return;
            }

            if (!IG || Me.ForcePercent > 30f)
            {

                if (MercCast("Electrocute", (TC || TD <= .5f) && StrongOrGreater(CT))) return;

                if (MercCast("Jolt", TC && StrongOrGreater(CT))) return;
                if (MercCast("Shock", TC && StrongOrGreater(CT))) return;

                if (MercCastOnGround("Death Field", StrongOrGreater(CT) || TWR30 >= 3, CT)) return;

                if (MercCast("Force Slow", StrongOrGreater(CT) && !CTDebuffs.Contains("Slow (Force)"))) return;

                if (MercCastOnGround("Force Storm", TWR30 >= 3, CT)) return;
                if (MercCast("Force Lightning")) return;
                if (MercCast("Crushing Darkness", StrongOrGreater(CT) && !CTDebuffs.Contains("Crushing Darkness (Force)"))) return;

            }

            if (MercCast("Saber Strike")) return;

        }
		
		
		
		
		
		
		
		
		

        public static void CorruptionCombat()
        {
            try
            {
                if (PvP) { CorruptionCombatPvP(); return; }

                WaitForCast();
                datLCL = DateTime.Now;

                Thread.Sleep(200);
                LoadVars();
                if (CT == null) return;
                if (TD >= 3.0f) return;

                if (!TargetInCombat()) return;

                if (MercCast("Mark of Power", !MyBuffs.Contains("Mark of Power"))) return;
                if (!IG) MoveTo(CT, 2.8f, "Force Speed");
                //if (!IG) MoveIfNec(.49f);

                if (!IG && BuddyTor.Me.IsMoving) StopMoving();

                if (MercCast("Unbreakable Will", Me.IsStunned)) return;

                if (MercCast("Static Barrier", Me, !MyBuffs.Contains("Static Barrier"))) return;

                if (MercCast("Resurgence", Me, !MyBuffs.Contains("Resurgence") && HP <= 70f)) return;
                if (MercCast("Unnatural Preservation", Me, HP <= 70f)) return;
                if (MercCastOnGround("Revivification", HP <= 70f, Me)) return;
                if (MercCast("Dark Heal", Me, HP <= 70f)) return;
                if (MercCast("Dark Infusion", Me, HP <= 70f)) return;


                if (IG) if (CorruptionHeals(85)) return;
                if (CorruptionCheckCompHealth()) return;

                if (MercCast("Recklessness", Me, !IG && StrongOrGreater(CT))) return;

                //if (MercCast("Cloud Mind", Me, CTT != CombatToughness.Player && HP <= 70f)) return;

                //if (MercCastOnGround("Death Field", HP <= 50, CT)) return;

                //if (MercCast("Affliction", !CTDebuffs.Contains("Affliction (Force)"))) return;
                //if (MercCast("Creeping Terror")) return;

                if (!PvP) if (MercCast("Crushing Darkness", !IG && !CT.InCombat && StrongOrGreater(CT) && !CTDebuffs.Contains("Crushing Darkness (Force)"))) return;

                if (MercCast("Recklessness", Me.ForcePercent <= 50f)) return;
                if (MercCast("Heroic Moment: Channel the Force", Me.ForcePercent <= 30f)) return;

                if (MercCast("Consumption", HP > 50 && Me.ForcePercent <= 20)) return;
                //if (SeerHeals(75f)) return;

                if (TD <= .50f)
                {
                    if (MercCast("Overload", TD <= .50f)) return;
                    if (MercCast("Tumult")) return;
                    if (MercCast("Electrocute")) return;
                    //if (MercCast("Double Strike")) return;
                }

                if (!IG || Me.ForcePercent > 30f)
                {

                    if (MercCast("Electrocute", (TC || TD <= .5f) && StrongOrGreater(CT))) return;
                    if (MercCast("Jolt", TC && StrongOrGreater(CT))) return;

                    if (!PvP) if (MercCast("Crushing Darkness", StrongOrGreater(CT) && !CTDebuffs.Contains("Crushing Darkness"))) return;
                    if (MercCast("Affliction", !CTDebuffs.Contains("Affliction"))) return;
                    if (MercCast("Shock")) return;
                    //if (MercCast("Lightning Strike")) return;

                    //if (MercCastOnGround("Death Field", StrongOrGreater(CT) || Helpers.Targets.Count(t => t.Distance <= 2.8f) >= 3, CT)) return;

                    if (MercCast("Force Slow", StrongOrGreater(CT) && !CTDebuffs.Contains("Slow (Force)"))) return;

                    //if (MercCast("Force Lightning")) return;
                }
                if (MercCastOnGround("Force Storm", TWR10 >= 3, CT)) return;
                if (MercCast("Force Lightning")) return;
                //if (MercCastOnGround("Force Storm", true, CT)) return;

                if (MercCast("Thrash", TD <= .50f)) return;
                if (MercCast("Saber Strike", TD <= .50f)) return;
                if (MercCast("Lightning Strike")) return;
                //Buddy.CommonBot.AbilityManager.Cast("Force Lightning", CT);   // Forced Cast - CanCast gets just a little flaky
            }
            catch { }
        }

        public static void CorruptionCombatPvP()
        { // Tried and tried on this - BW seems just about uselesss with it due to all of the bugs and hangs..... 
                try
                {
                    Thread.Sleep(300);

                    IG = true;

                    if (Me.IsDead || Me.HealthPercent < .2f) return;

                    WaitForCast();
                    datLCL = DateTime.Now;

                    //LoadVars();
                    ///if (!TargetInCombat()) return;

                    //try { Me.EnemiesAttackers.OrderBy(t => t.Distance).FirstOrDefault().Target(); }
                    //catch { }
                    CT = Me.CurrentTarget;
                    if (CT == null) return;
                    TD = CT.Distance;
                    if (TD >= 3.0f) return;
                    HP = Me.HealthPercent;
                    RS = Me.ResourceStat;

                    MyBuffs = "";
                    foreach (TorEffect te in Me.Buffs) MyBuffs += te.Name + ";";
                    CTDebuffs = "";
                    if (CT != null) foreach (TorEffect te in CT.Debuffs) CTDebuffs += te.Name + ";";

                    MySpec = SkillTreeId.SorcererCorruption;
                    //CT.Face();

                    if (TD <= .50f)
                    {
                        if (MercCast("Overload", TD <= .50f)) return;
                        if (MercCast("Tumult")) return;
                        if (MercCast("Electrocute")) return;
                        //if (MercCast("Double Strike")) return;
                    }

                    if (MercCast("Mark of Power", !MyBuffs.Contains("Mark of Power"))) return;
                    if (!IG) MoveTo(CT, 2.8f, "Force Speed");
                    //if (!IG) MoveIfNec(.49f);

                    if (!IG && BuddyTor.Me.IsMoving) StopMoving();
                    if (MercCast("Static Barrier", Me, !MyBuffs.Contains("Static Barrier"))) return;

                    if (MercCast("Unbreakable Will", Me.IsStunned)) return;

                    if (MercCast("Unnatural Preservation", Me, HP <= 70f)) return;
                    if (MercCast("Resurgence", Me, MyBuffs.Contains("Resurgence") && HP <= 70f)) return;
                    if (MercCast("Dark Infusion", Me, HP <= 70f)) return;
                    if (MercCast("Dark Heal", Me, HP <= 70f)) return;

                    if (CorruptionHeals(80f)) return;

                    if (MercCast("Recklessness", Me, !IG && StrongOrGreater(CT))) return;

                    if (MercCast("Recklessness", Me.ForcePercent <= 50f)) return;
                    if (MercCast("Heroic Moment: Channel the Force", Me.ForcePercent <= 30f)) return;

                    if (MercCast("Consumption", HP > 50 && Me.ForcePercent <= 20)) return;
                    //if (SeerHeals(75f)) return;

                    if (TD <= .50f)
                    {
                        if (MercCast("Overload", TD <= .50f)) return;
                        if (MercCast("Tumult")) return;
                        //if (MercCast("Double Strike")) return;
                    }

                    if (!IG || Me.ForcePercent > 30f)
                    {

                        if (MercCast("Electrocute", (TC || TD <= .5f) && StrongOrGreater(CT))) return;
                        if (MercCast("Jolt", TC && StrongOrGreater(CT))) return;

                        if (MercCast("Affliction", !CTDebuffs.Contains("Affliction"))) return;
                        if (MercCast("Shock")) return;

                        if (MercCast("Force Slow", StrongOrGreater(CT) && !CTDebuffs.Contains("Slow (Force)"))) return;
                    }
                    if (MercCast("Force Lightning")) return;

                    if (MercCast("Thrash", TD <= .50f)) return;
                    if (MercCast("Saber Strike", TD <= .50f)) return;
                }
                catch { }
                if (DateTime.Now.Subtract(datLCs).TotalSeconds >= 4) MercForcedCast("Lightning Strike", CT);
        }

        public static void WatchmanCombat()
        {

            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(100);
            LoadVars();

            if (MercCast("Juyo Form", !MyBuffs.Contains("Juyo Form"))) return;
            if (MercCast("Force Might", !MyBuffs.Contains("Force Might"))) return;

            if (!TargetInCombat()) return;

            MoveTo(CT, .490f, "Force Leap");

            //if (!IG) MoveIfNec(.49f);

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();

            if (MercCast("Resolute", Me, Me.IsStunned)) return;
            if (MercCast("Saber Ward", Me, HP <= 50)) return;


            if (MercCast("Rebuke", HP <= 70)) return;
            if (MercCast("Zen", Me, HP <= 70f)) return;

            if (MercCast("Overload Saber", Me)) return;
            if (MercCast("Zealous Strike")) return;
            if (MercCast("Cauterize")) return;
            if (MercCast("Merciless Slash")) return;
            if (MercCast("Master Strike")) return;

            if (MercCast("Dispatch")) return;

            if (MercCast("Riposte")) return;
            if (MercCast("Force Kick", TD <= .50f)) return;
            if (MercCast("Crippling Throw")) return;

            if (MercCast("Blade Storm", TWR10 >= 2 || StrongOrGreater(CT))) return;
            if (MercCastOnGround("Force Sweep", TWR5 >= 3 || StrongOrGreater(CT), CT)) return;

            if (MercCast("Master Strike")) return;
            if (MercCast("Force Leap")) return;
            if (MercCast("Slash")) return;

            if (MercCast("Slash")) return;
            if (MercCast("Strike")) return;
            

            return;

        }

        public static void DarknessCombat()
        {

            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(200);
            LoadVars();

            if (CT == null && Me.CurrentTarget != null) CT = Me.CurrentTarget;

            if (!CT.InCombat && CT.HealthPercent > .2f)
            {
                MercCast("Stealth");
                while (CT.Distance > .15f) MoveTo(CT, .15f);
                if (MercCast("Maul", CT.Distance <= .20f)) if (MercCast("Thrash")) return;
            }
            //if (!TargetInCombat()) return;

            if (MercCast("Juyo Form", !MyBuffs.Contains("Juyo Form"))) return;
            if (MercCast("Mark of Power", !MyBuffs.Contains("Mark of Power"))) return;
            if (MercCast("Surging Charge", !MyBuffs.Contains("Surging Charge"))) return;

            if (!CT.InCombat && Me.IsStealthed) return;

            //if (TD >= .50f && TD <= 3.0f) if (MercCast("Force Leap")) return;

            if (MercCast("Unbreakable Will", Me.IsStunned)) return;

            if (TC)
            {
                if (MercCast("Electrocute")) return;
                if (MercCast("Shock")) return;
            }

            if (!IG) MoveTo(CT, .490f, "Shock");

            if (MercCast("Overcharge Saber", HP <= 70f)) return;
            if (MercCast("Force Shroud", HP <= 40f)) return;
            if (MercCast("Deflection", HP <= 50f)) return;

            if (MercCast("Dark Ward")) return;
            if (MercCast("Wither", TWR10 >= 2)) return;
            //if (MercCast("Dark Charge")) return;
            if (MercCast("Discharge", MyBuffs.Contains("Dark Charge"))) return;
            if (MercCast("Force Lightning", MyBuffs.Contains("Harnessed Darkness"))) return;
            if (MercCast("Assassinate", THP <= 30f)) return;

            if (MercCast("Lacerate", TWR10 >= 3)) return;

            if (MercCast("Shock")) return;
            if (MercCast("Force Lightning")) return;

            if (StrongOrGreater(CT))
            {
                if (MercCast("Recklessness")) return;
                if (MercCast("Electrocute")) return;
                if (MercCast("Force Slow")) return;
            }

            //if (!IG) MoveIfNec(.49f);

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();
            if (MercCast("Overload", TD <= .50f)) return;
            if (MercCast("Thrash")) return;
            if (MercCast("Saber Strike")) return;

        }

        public static void DefenseCombat()
        {

            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(200);
            LoadVars();

            if (!TargetInCombat()) return;

            if (CT == null && Me.CurrentTarget != null) CT = Me.CurrentTarget;

            if (!IG) StopMoving();

            MoveTo(CT, .40f, "Force Leap");

            if (MercCast("Resolute", Me.IsStunned)) return;

            if (!MyBuffs.Contains("Soresu Form")) if (AbilityManager.CanCast("Soresu Form", Me)) if (MercCast("Soresu Form", Me)) return;
            if (!MyBuffs.Contains("Shii-Cho Form")) if (!AbilityManager.CanCast("Soresu Form", Me)) if (MercCast("Shii-Cho Form", Me)) return;

            if (MercCast("Hilt Strike", CT.IsStunned && RS >= 3)) return;
            if (MercCast("Pommel Strike", CT.IsStunned)) return;

            if (MercCast("Dispatch", HP <= 20 && RS >= 3)) return;
            if (MercCast("Hilt Strike", TC && HP <= 30f && MyBuffs.Contains("Saber Ward"))) return;
            if (MercBuffSelf("Enure", HP <= 30)) return;

            if (MercBuffSelf("Saber Ward", HP <= 50)) return;
            if (MercCast("Awe", TWR5 >= 3 && HP <= 60)) return;
            if (MercCast("Force Stasis", TC && HP <= 60f && MyBuffs.Contains("Saber Ward"))) return;
            if (MercBuffSelf("Warding Call", HP <= 60)) return;

            if (MercCast("Sundering Strike")) return;
            if (MercCast("Master Strike")) return;
            if (MercCast("Freezing Force", !CTDebuffs.Contains("Slowed (Force)") && (TWR5 >= 2 || StrongOrGreater(CT)))) return;
            if (MercCast("Blade Storm")) return;

            if (MercCast("Force Sweep", TWR5 >= 2 && RS >= 3)) return;
            if (MercCast("Guardian Slash", RS >= 4 && TWR10 >= 3)) return;
            if (MercCast("Cyclone Slash", TWR5 >= 4)) return;
            if (MercCast("Force Push", TWR10 >= 3)) return;

            if (MercCast("Saber Throw", TD >= .50f)) return;
            if (MercCast("Combat Focus", RS <= 6)) return;

            if (MercCast("Force Kick", TC)) return;

            if (MercCast("Opportune Strike", CTT != CombatToughness.Player)) return;


            if (MercCast("Slash", RS >= 6)) return;
            if (MercCast("Strike")) return;

        }

        public static void CheckForGuarding()
        {
            NGUnit = null;
            NAUnit = null;

            foreach (TorNpc tn in Helpers.Targets.OrderBy(t => t.Distance).Where(t=>!t.IsDead && t.CurrentTarget != Me))
                try
                {
                    bool Att = false;
                    if (tn != null) if (!tn.IsDeleted) if (tn.InCombat)
                            {
                                Att = (tn.CurrentTarget.Toughness == CombatToughness.Player);
                                if (!Att) if (Comp != null) if (tn.CurrentTarget == Comp) Att = true;
                                if (Att)
                                {
                                    TorCharacter pt = tn.CurrentTarget;
                                    if (pt.HealthPercent < 80) { NGUnit = pt; NAUnit = tn; break; }
                                }
                            }
                }
                catch { }
        }

        public static bool HandleGuardingJugg()
        {
            
            //if (!IG) return false;

            if(IG) CheckForGuarding();

            if (NAUnit != null) try
                {
                    CT = NAUnit;

                    if (MercCast("Taunt", NAUnit)) return true;
                    if (MercCast("Intercede", NGUnit)) return true;
                    if (MercCast("Saber Throw", NAUnit)) return true;
                    if (MercCast("Guard", NGUnit)) return true;

                }
                catch { }
            return false;
        }

        public static void JuggernautCombat()
        {

            WaitForCast();
            datLCL = DateTime.Now;

            Thread.Sleep(100);
            LoadVars();

            if (CT == null && Me.CurrentTarget != null) CT = Me.CurrentTarget;

            if (MercCast("Soresu Form", !MyBuffs.Contains("Soresu Form"))) return;
            //if (MercCast("Shi-Cho Form", !MyBuffs.Contains("Shii-Cho Form"))) return;
            if (MercCast("Unnatural Might", !MyBuffs.Contains("Unnatural Might"))) return;

            if (MercCast("Force Scream", TD <= 1.0f)) return;

            if (!IG) MoveTo(CT, .490f, "Force Charge");

            if (!IG && BuddyTor.Me.IsMoving) StopMoving();

            if (MercCast("Unleash", BuddyTor.Me.IsStunned)) return;
            if (MercCast("Disruption", TC)) return;
            if (MercCast("Saber Ward", HP <= 50f)) return;
            if (MercCast("Invincible", HP <= 50f)) return;
            if (MercCast("Endure Pain", HP <= 50)) return;
            if (MercCast("Force Push", (TWR10 >= 3))) return;

            if (MercCast("Guard", Comp, CHP <= 50)) return;
            HandleGuardingJugg();

            if (MercCast("Savage Kick")) return;
            if (MercCast("Sundering Assault")) return;
            if (MercCast("Retaliation")) return;
            if (MercCast("Ravage")) return;

            if (MercCast("Enrage")) return;
            if (MercCast("Crushing Blow")) return;
            if (MercCast("Backhand")) return;
            if (MercCast("Sweeping Slash", (TWR5 >= 3))) return;

            if (MercCast("Smash")) return;

            if (MercCast("Vicious Throw", THP <= 30f)) return;
            if (MercCast("Pommel Strike", CT.IsStunned)) return;
            if (MercCast("Force Choke")) return;


            if (MercCast("Deadly Throw")) return;
            if (MercCast("Intimidating Roar")) return;

            if (MercCast("Chilling Scream", !CTDebuffs.Contains("Slow (Force)") && (HP <= 50 || TWR10 >= 2))) return;
            if (MercCast("Threatening Scream", (TWR10 >= 3))) return;

            if (MercCast("Vicious Slash")) return;
            if (MercCast("Assault")) return;

        }

        public static void TargetingToggle()
        {
            AlwaysTargetWeakest = !AlwaysTargetWeakest;
            Logging.Write("Target-Weakest Toggled " + (AlwaysTargetWeakest?"ON":"OFF"));
        }

        // Credit to ChinaJade and WingIt for this routine
        public static void MovementToggle()
        {
            //if (!IG)
            //{
            //    Logging.Write("Key to disable movement only valid when In-Group - set to ON.");
            //    MovementAllowed = true;
            //    //return;
            //}
            //else
            {
                MovementAllowed = !MovementAllowed;
                Logging.Write("Movement Toggled " + (MovementAllowed ? "ON" : "OFF"));
            }
            
            if (!MovementAllowed)
            {
                Buddy.Navigation.Navigator.NavigationProvider = NoNavigationProvider;
                Buddy.Navigation.Navigator.MovementProvider = NoMovementProvider;
            }
            else
            {
                Buddy.Navigation.Navigator.NavigationProvider = DefaultNP;
                Buddy.Navigation.Navigator.MovementProvider = DefaultMP;
            }

        }

        public static void Inits()
        {
            //Logging.Write("Checking Abilities...");
            if (!InitsComplete)
            {
                MyBlacklist = "";
                datLLSl = DateTime.Now;

                //Buddy.CommonBot.Settings.CharacterSettings.PullDistance = PullDist;
                foreach (Buddy.Swtor.Objects.Components.ProfessionInfo pi in BuddyTor.Me.ProfessionInfos)
                {
                    Logging.Write("Abil " + pi.Name + " found");
                    if (pi.Name.Contains("Bioanalysis")) HasBio = true;
                    if (pi.Name.Contains("Scavenging")) HasScav = true;
                    if (pi.Name.Contains("Archaeology")) HasArch = true;
                    if (pi.Name.Contains("Slicing")) HasSlicing = true;
                }

                CT = null;
                GS.PullRange = Math.Max(1.50f, PullDist);

                Logging.Write("Pull Range set at " + GS.PullRange.ToString());

                Logging.Write("Movement Toggle Key [" + MovementToggleHotKey.ToString() + "] HotKey Registered...  You can edit the value in MercHelpers.cs if necessary (and then restart the .exe)");
                Hotkeys.RegisterHotkey("Movement Toggle", () => { MovementToggle(); }, MovementToggleHotKey);
                Logging.Write("Target-Weakest Toggle Key [" + TargWeakestToggleHotKey.ToString() + "] HotKey Registered...  You can edit the value in MercHelpers.cs if necessary (and then restart the .exe)");
                Hotkeys.RegisterHotkey("Targeting Toggle", () => { TargetingToggle(); }, TargWeakestToggleHotKey);
                //Logging.Write("Forced-Combat Key [" + ForceCombatHotKey.ToString() + "] HotKey Registered (to FORCE Combat if hung)...");
                //Hotkeys.RegisterHotkey("Movement Toggle", () => { OOC(); }, ForceCombatHotKey);
                //Hotkeys.RegisterHotkey("Force Routine Reload", () => { DoRestart(); }, Keys.OemPipe);

                //foreach (TorAbility ta in BuddyTor.Me.KnownAbilitiesContainer)
                //{
                //    //Logging.Write(ta.Name);

                //    if (ta.Name.Contains("Stealth")) taStealth = ta;
                //    if (ta.Name.Contains("Attack")) Logging.Write("Attack Ability Found. (for Companion)");
                //    if (ta.Name.Contains("Jab")) taJab = ta;
                //    if (ta.Name.Contains("Uppercut")) taUppercut = ta;
                //    if (ta.Name.Contains("Bash")) taBash = ta;
                //    if (ta.Name.Contains("Unity")) taUnity = ta;
                //}

                MySpec = SpecHandler.GetSpec();

                if (DoRandomGrind)
                {
                    Logging.Write("RANDOM-GRIND IS ACTIVE.  If not intended, update the DoRandomGrind flag in MercHelpers.cs to false, and restart the .exe (as in exit and reopen).");
                    Logging.Write("WITH DoRandomGrind, YOU NEED TO PARK YOUR CHARACTER IN A MOB-RICH AREA BEFORE STARTING (So it has something to find/attack).");
                }
                Logging.Write("Check Abilities: Bio " + HasBio.ToString() + " Scav " + HasScav.ToString() + " Arch " + HasArch.ToString() + " Slicing " + HasSlicing.ToString());
                InitsComplete = true;
                ForcePulse();
            }
        }

        private static bool HandleDebuffs()
        {
            return HandleDebuffs(Me);
        }

        private static bool HandleDebuffs(TorCharacter unit)
        {
            if (unit == null) return false;

            foreach (TorEffect te in unit.Debuffs)
            {
                string spellCure = "";
                if (MySpec == SkillTreeId.MercenaryBodyguard) spellCure = "Cure";
                else if (MySpec == SkillTreeId.CommandoCombatMedic) spellCure = "Field Aid";
                else if (MySpec == SkillTreeId.SorcererCorruption) spellCure = "Purge";
                else { return false; }

                if (te.Name.Contains("Burning")) if (MercCast(spellCure, unit)) return true;
                //if (te.Name.Contains("Armor")) if (MercCast(spellCure, unit)) return true;  // Can't be dispelled
                if (te.Name.Contains("Slowed")) if (MercCast(spellCure, unit)) return true;
                if (te.Name.Contains("Shocked")) if (MercCast(spellCure, unit)) return true;
                if (te.Name.Contains("Poisoned")) if (MercCast(spellCure, unit)) return true;
                if (te.Name.Contains("Crippling")) if (MercCast(spellCure, unit)) return true;
                if (te.Name.Contains("Crush")) if (MercCast(spellCure, unit)) return true;
                if (te.Name.Contains("Infected")) if (MercCast(spellCure, unit)) return true;
                if (te.Name.Contains("Bleeding")) if (MercCast(spellCure, unit)) return true;
                if (te.Name.Contains("Grav")) if (MercCast(spellCure, unit)) return true;
                if (te.Name.Contains("Hallucinat")) if (MercCast(spellCure, unit)) return true;
            }
            return false;
        }

        public static bool MercCastOnGround(string theSpell, bool requirements, TorCharacter unit)
        {
            if (unit == null) return false;
            if (theSpell == "") return false;
            return MercCastOnGround(theSpell, requirements, unit, false);
        }

        public static bool MercCastOnGround(string theSpell, bool requirements, TorCharacter unit, bool OnlyInCombat)
        {
            if (unit == null) return false;
            if (theSpell == "") return false;
            if (unit.IsDead) return false;

            try
            {
                if (requirements) if (Buddy.CommonBot.AbilityManager.CanCast(theSpell, unit))
                    {
                        datLCs = DateTime.Now;

                        Logging.Write("RS: " + RS.ToString("000.0") + " THP: " + CT.HealthPercent.ToString("00.00") + " Casting " + theSpell);
                        AbilityManager.Cast(theSpell, unit.Position);
                        Thread.Sleep(500);
                        if (OnlyInCombat)
                        {
                            while (Me.InCombat && Me.IsCasting) Thread.Sleep(500);
                            return true;
                        }
                        WaitForCast();
                        return true;
                    }
            }
            catch { }
            return false;
        }

        // Used in 'Directional Casting' such as Pulse Cannon - WE HAVE TO BE FACING THE TARGET FIRST - otherwise - useless
        public static bool MercFaceAndCast(string theSpell, bool req)
        {
            if (CT == null) return false;
            if (theSpell == "") return false;

            if (req)
            {
                CT.Face();          // Bug w/Face() - do it twice to be safe
                CT.Face();
                Thread.Sleep(300);

                if (MercCast(theSpell, TD <= 1.0f)) return true;
            }
            return false;
        }

        public static bool MercBuffSelf(string theSpell, bool requirements)
        {
            if (requirements && !Me.HasBuff(theSpell)) if (MercCast(theSpell, Me)) return true;
            return false;
        }

        public static bool MercCast(string theSpell, bool requirements)
        {
            if (requirements) return MercCast(theSpell);
            return false;
        }

        public static bool MercCast(string theSpell, TorCharacter theChar, bool requirements)
        {
            if (theChar == null) return false;
            if (theChar.IsDead) return false;
            if (requirements) return MercCast(theSpell, theChar);
            return false;
        }

        public static bool MercCast(string theSpell)
        {
            //return MercCast(theSpell, BuddyTor.Me.CurrentTarget);
            if (CT == null) return false;
            if (CT.IsDead) return false;
            return MercCast(theSpell, CT);
        }

        public static bool MercCast(string theSpell, TorCharacter theChar)
        {
            try
            {

                if (theChar == null) return false;
                if (theSpell == "") return false;
                if (theChar.IsDead) return false;
                if (Buddy.CommonBot.AbilityManager.CanCast(theSpell, theChar))
                {
                    datLCs = DateTime.Now;
                    Buddy.CommonBot.AbilityManager.Cast(theSpell, theChar);
                    Logging.Write("RS: " + RS.ToString("000.0") + " THP: " + CT.HealthPercent.ToString("00.00") + " Casting " + theSpell);
                    Thread.Sleep(200);
                    WaitForCast();
                    return true;
                }
            }
            catch { }
            return false;
        }

        public static void MercForcedCast(string theSpell, TorCharacter theUnit)
        {
            try
            {
                if (theSpell == "") return;
                datLCs = DateTime.Now;
                Buddy.CommonBot.AbilityManager.Cast(theSpell, theUnit);
            }
            catch { }
        }

        static bool HaveTarget()
        {
            if (CT != null) if (!CT.IsFriendly && !CT.IsDead) return true;
            return false;
        }

        static void CheckForGroup()
        {
            //Sketchy trying to determine 'Group Flag'
            //IG = Global.IsInGroup && BuddyTor.Me.GroupId != 0 && BuddyTor.Me.GroupId != null;
            //IG = (BuddyTor.Me.GroupId != 0 && BuddyTor.Me.GroupId != null) || (BuddyTor.Me.WarzoneTeam!=WarzoneTeam.None);
            if (PvP) 
                IG = true;
            else
                IG = (!PullIG && BuddyTor.Me.GroupId != 0);
        }

        public static void GetEffects()
        {
            MyBuffs = "";  // Me.HasBuff(X) will ignore element 0 - buggy, so created this
            try {foreach (TorEffect te in Me.Buffs) MyBuffs += te.Name + ";";} catch { }
            CTDebuffs = "";
            try {if (CT != null) foreach (TorEffect te in CT.Debuffs) CTDebuffs += te.Name + ";";} catch { }
        }

        public static void LoadVars()
        {
            if (PvP) IG = true;

            //CT = null;
            if (Me.CurrentTarget != null) UpdateTarget(); else CT = Me.CurrentTarget;

            GetEffects();            
            //Logging.Write("Buffs: " + MyBuffs);
            //Logging.Write(Buddy.CommonBot.AbilityManager.CanCast("Stealth", Me).ToString());

            try
            {
                //CT = null;
                //Comp = null;
                CompTarg = null;

                TD = 100;
                RS = 50;
                TC = false;
                LHH = 0;
                CTT = CombatToughness.None;
                LHD = 100;
                THP = 0;
                TC = false;
                THP = 100;

                //HP = BuddyTor.Me.HealthPercent;
                HP = (float)BuddyTor.Me.HealthPercent;
                EP = (float)BuddyTor.Me.EnergyPercent;
                //CT = BuddyTor.Me.CurrentTarget;

                if (CT != null) if (CT.Guid != CG)
                    {
                        if (CG != CT.Guid) Logging.Write("Initiating Combat with " + CT.Name + " InGroup: " + IG.ToString());
                        CG = (ulong)CT.Guid;
                        CS = DateTime.Now;
                    }

                if (CT != null)
                    try
                    {
                        CTT = CombatToughness.None;

                        if (PvP) CTT = CombatToughness.Player; else CTT = CT.Toughness;
                        //CTT = CT.Toughness;
                        //TD = BuddyTor.Me.CurrentTarget.Distance;
                        TD = BuddyTor.Me.Position.Distance(CT.Position);
                        TC = CT.IsCasting;
                        THP = CT.HealthPercent;
                    }
                    catch { }

                RS = (float)BuddyTor.Me.ResourceStat;
                if (!PvP) CheckForGroup();

                CompTarg = null;
                //Comp = BuddyTor.Me.Companion;

                CHP = 100;
                if (!PvP) if (Comp != null)
                {
                    if (!Comp.IsDead) if (Comp.CurrentTarget != null) if (!Comp.CurrentTarget.IsFriendly) CompTarg = Comp.CurrentTarget;
                    //CHP = (float)Comp.HealthPercent;
                }
                //CheckForHeals();
                TWR30 = 0;
                TWR10 = 0;
                TWR5 = 0;

                if (!PvP && Me.InCombat)
                {
                    try
                    {
                        TWR30 = Helpers.Targets.Where(t => !t.IsDead && t.Distance <= 3.0f).Count();
                        TWR10 = Helpers.Targets.Where(t => !t.IsDead && t.Distance <= 1.0f).Count();
                        TWR5 = Helpers.Targets.Where(t => !t.IsDead && t.Distance <= .50f).Count();
                    }
                    catch { }
                }
                if (PvP && Me.InCombat) try
                    {

                        TWR30 = Me.EnemiesAttackers.Where(t => !t.IsDead && t.Distance <= 3.0f).Count();
                        TWR10 = Me.EnemiesAttackers.Where(t => !t.IsDead && t.Distance <= 1.0f).Count();
                        TWR5 = Me.EnemiesAttackers.Where(t => !t.IsDead && t.Distance <= .50f).Count();
                    }
                    catch { }
            }
            catch { Logging.Write("Error Setting Values."); }
            //Comp = null;
            //return false;
        }

        public static TorCharacter Me { get { return BuddyTor.Me; } }
        //public static TorCharacter Comp { get { return BuddyTor.Me.Companion; } }

        public static TorCharacter Comp
        {   

            //if (DateTime.Now.Subtract(datLCC).TotalSeconds >= 5)
            get
            {
                if (PvP) return null;
                
                if (BuddyTor.Me.IsMounted) { LastComp = null; return null; }
                if (!IG) ForcePulse();
                if (BuddyTor.Me.Companion != null) { LastComp = BuddyTor.Me.Companion; CHP = (float)LastComp.HealthPercent; return LastComp; }

                bool bErr = false;
                if (LastComp != null) try
                    {
                        float HealthTest = LastComp.HealthPercent;
                    }
                    catch { bErr = true; }

                if (!bErr) return LastComp;

                if (LastComp != null && DateTime.Now.Subtract(datLCC).TotalSeconds < 10) return LastComp;

                datLCL = DateTime.Now;
                foreach (TorNpc mob in ObjectManager.GetObjects<TorCharacter>().OrderBy(t => t.Distance).Where(g=>(g.Toughness==CombatToughness.Companion || g.Guid==BuddyTor.Me.CompanionGuid)))
                    try
                    {
                        if (mob != null) //if (!mob.IsDeleted) 
                        { 
                            //Logging.Write("Get Comp Dur: " + DateTime.Now.Subtract(datLCL).TotalSeconds.ToString("0.0")); 
                            //Logging.Write("Comp FOUND."); 
                            CHP = (float)mob.HealthPercent;
                            datLCC = DateTime.Now; LastComp = mob;
                            return mob; 
                        }
                    }
                    catch { return null; }
                //Logging.Write("Comp NOT found."); 
                return null;
            }
        }

        public static bool IsBG { get { return BuddyTor.Me.AdvancedClass == Buddy.Swtor.AdvancedClass.Mercenary; } }
        public static bool IsCM { get { return BuddyTor.Me.AdvancedClass == Buddy.Swtor.AdvancedClass.Commando; } }
        public static bool IsSage { get { return BuddyTor.Me.AdvancedClass == Buddy.Swtor.AdvancedClass.Sage; } }
        //public static float CHP { get { return Comp == null ? 100 : (float)Comp.HealthPercent;} }

        public static void CheckForHeals(float minH)
        {
            LHUnit = null;
            LHUnit = UnitLowOnHealth(minH);
            if (LHUnit != null)
            {
                LHH = (float)LHUnit.HealthPercent;
                LHD = (float)LHUnit.Distance;
                LHP = (Vector3)LHUnit.Position;
            }
        }

        // Buff nearby players if in-range
        public static bool CheckPlayerBuffs(string Buff)
        {
            if (BuddyTor.Me.IsMounted) return false;
            if (DateTime.Now.Subtract(datLBC).TotalSeconds < 20) return false;
            try
            {
                foreach (TorPlayer unit in ObjectManager.GetObjects<TorPlayer>().OrderBy(t => t.HealthPercent))
                    if (unit.Distance <= 2.8f && unit != Me && unit.IsFriendly && unit.InLineOfSight && !unit.HasBuff(Buff))
                    {datLBC = DateTime.Now; MercCast(Buff, unit); Thread.Sleep(500); return true;}
            }
            catch { 
                //Logging.Write("Error in CheckPlayerBuffs()"); 
                return false; 
            }
            return false;
        }

        public static TorPlayer UnitLowOnHealth(float minH)
        {
            for (int inti = 0; inti < 3; inti++)
            {
                try
                {
                    //foreach (TorCharacter unit in ObjectManager.GetObjects<TorCharacter>().OrderBy(t => t.HealthPercent))
                    foreach (TorPlayer unit in ObjectManager.GetObjects<TorPlayer>())
                        if ((!unit.IsHostile || unit.IsFriendly) && unit.Distance <= 2.8f && unit.HealthPercent < minH )
                        {
                            float uhp = (float)unit.HealthPercent;
                            //Logging.Write("Unit: " + unit.Name + " HP: " + uhp.ToString());
                            if (uhp < 1.0f)
                            {
                                if (MySpec == SkillTreeId.SorcererCorruption) MercCast("Reanimation", unit);
                                else if (MySpec == SkillTreeId.OperativeMedic) MercCast("Resuscitation Probe", unit);
                                else if (MySpec == SkillTreeId.MercenaryBodyguard) MercCast("Onboard AED", unit);
                                else if (MySpec == SkillTreeId.SageSeer) MercCast("Revival", unit);
                                else if (MySpec == SkillTreeId.CommandoCombatMedic) MercCast("Emergency Medical Probe");
                            }                            //else
                                if (uhp <= minH && uhp >= 1.0f)
                                    return unit;
                        }
                }
                catch {}
                Thread.Sleep(50);
            }
            return null;
        }

        // VERY unreliable - Not Used
        ////public static TorPlayer PNH(Single minH)
        ////{
        ////    TorPlayer LHPC=null;
        ////    try
        ////    {
        ////        if (Global.IsInGroup) if (Helpers.Group != null) if (Helpers.LowestHealthGroupPlayer != null)
        ////                    if (Helpers.LowestHealthGroupPlayer.IsFriendly && Helpers.LowestHealthGroupPlayer.Distance <= 3.0f && Helpers.LowestHealthGroupPlayer.HealthPercent <= minH) LHPC = Helpers.LowestHealthGroupPlayer;
        ////    }
        ////    catch { return null; }
        ////    if (LHPC.IsDead) LHPC = null;
        ////    return LHPC;
        ////}
        public static bool MercMoveAndStop(TorCharacter tc, float rng)
        {
            CommonBehaviors.MoveAndStop(ret => tc.Position, rng, true, "Target Position");
            return false;

        }

        public static bool BGCheckCompHealth()
        {
            try
            {

                //Comp = BuddyTor.Me.Companion;
                if (Comp == null) return false;

                //CHP = (float)BuddyTor.Me.Companion.HealthPercent;

                if (CHP <= 50)
                {
                    WaitForCast();
                    if (Comp.Distance > 2.8f) { MoveTo(Comp, 2.8f); Thread.Sleep(300); }
                    if (CHP < 1f)
                    {
                        if (MercCast("Onboard AED", Comp)) return true;
                    }
                    else
                    {
                        if (MercCastOnGround("Kolto Missile", true, Comp)) return true;
                        if (MercCast("Emergency Scan", Comp)) return true;
                        if (MercCast("Healing Scan", Comp)) return true;
                        if (!Comp.HasBuff("Kolto Shell")) if (MercCast("Kolto Shell", BuddyTor.Me.Companion)) return true;
                        if (MercCast("Rapid Scan", Comp)) return true;
                        if (MyBuffs.Contains("Combat Support Cylinder")) if (MercCast("Rapid Shots", Comp)) return true;
                    }
                }
            }
            catch { Logging.Write("Error in CheckCompHealth()"); }

            return false;
        }

        public static bool CMCheckCompHealth()
        {
            try
            {

                //Comp = BuddyTor.Me.Companion;
                if (Comp == null) return false;

                //CHP = (float)BuddyTor.Me.Companion.HealthPercent;
                HandleDebuffs(Comp);

                if (CHP <= 50)
                {
                    WaitForCast();
                    if (Comp.Distance > 2.8f) { MoveTo(Comp, 2.8f); Thread.Sleep(300); }
                    if (CHP < 1f)
                    {
                        if (MercCast("Onboard AED", Comp)) return true;
                    }
                    else
                    {
                        if (MercCastOnGround("Kolto Bomb", true, Comp)) return true;
                        if (MercCast("Advanced Medical Probe", Comp)) return true;
                        //if (!Comp.HasBuff("Trauma Probe")) if (MercCast("Trauma Probe", Comp)) return true;
                        if (MercCast("Medical Probe", Comp)) return true;
                        if (MyBuffs.Contains("Combat Support Cell")) if (MercCast("Hammer Shot", Comp)) return true;
                        //if (MercCast("Emergency Scan", Comp)) return true;
                        //if (MercCast("Healing Scan", Comp)) return true;
                        //if (!Comp.HasBuff("Kolto Shell")) if (MercCast("Kolto Shell", BuddyTor.Me.Companion)) return true;
                    }
                }
            }
            catch { Logging.Write("Error in CheckCompHealth()"); }

            return false;
        }

        public static bool MedicineCheckCompHealth()
        {
            try
            {

                //Comp = BuddyTor.Me.Companion;
                if (Comp == null) return false;

                //CHP = (float)BuddyTor.Me.Companion.HealthPercent;

                if (CHP <= 50)
                {
                    WaitForCast();
                    if (Comp.Distance > 2.8f) { MoveTo(Comp, 2.8f); Thread.Sleep(300); }
                    if (CHP < 1f)
                    {
                        //if (MercCast("Onboard AED", Comp)) return true;
                    }
                    else
                    {
                        if (MercCast("Surgical Probe", Comp, MyBuffs.Contains("Tactical Advantage"))) return true;
                        if (MercCast("Kolto Infusion", Comp, MyBuffs.Contains("Tactical Advantage"))) return true;
                        if (MercCast("Kolto Probe", Comp, HP <= 70 && !Comp.HasBuff("Kolto Probe"))) return true;
                        if (MercCast("Kolto Injection", Comp)) return true;
                        //if (MercCast("Stim Boost", Comp, MyBuffs.Contains("Tactical Advantage"))) return true;
                    }
                }
            }
            catch { Logging.Write("Error in CheckCompHealth()"); }

            return false;
        }

        public static bool SeerCheckCompHealth()
        {
            try
            {

                //Comp = BuddyTor.Me.Companion;
                if (Comp == null) return false;

                //CHP = (float)BuddyTor.Me.Companion.HealthPercent;
                if (!Comp.HasBuff("Force Armor")) if (MercCast("Force Armor", Comp)) return true;

                if (CHP <= 50)
                {
                    WaitForCast();
                    if (Comp.Distance > 2.8f) { MoveTo(Comp, 2.8f); Thread.Sleep(300); }
                    if (CHP < 1f)
                    {
                        if (MercCast("Onboard AED", Comp)) return true;
                    }
                    else
                    {
                        if (MercCast("Force Armor", Comp, !Comp.HasBuff("Force Armor"))) return true;

                        if (MercCast("Rejuvenate", Comp, !Comp.HasBuff("Rejuvenate"))) return true;
                        if (MercCastOnGround("Salvation", !Comp.HasBuff("Salvation"), Comp)) return true;
                        if (MercCast("Benevolence", Comp)) return true;
                        if (MercCast("Deliverance", Comp)) return true;
                        if (MercCast("Healing Trance", Comp, !Comp.HasBuff("Healing Trance"))) return true;

                        if (MercCast("Rescue", Comp, CHP <= 20)) return true;
                    }
                }
            }
            catch { Logging.Write("Error in CheckCompHealth()"); }

            return false;
        }

        public static bool CorruptionCheckCompHealth()
        {
            try
            {

                //Comp = BuddyTor.Me.Companion;
                if (Comp == null) return false;

                //CHP = (float)BuddyTor.Me.Companion.HealthPercent;
                if (!Comp.HasBuff("Static Barrier")) if (MercCast("Static Barrier", Comp)) return true;

                if (CHP <= 50)
                {
                    WaitForCast();
                    if (Comp.Distance > 2.8f) { MoveTo(Comp, 2.8f); Thread.Sleep(300); }
                    if (CHP < 1f)
                    {
                        if (MercCast("Reanimation", Comp)) return true;
                    }
                    else
                    {
                        if (MercCast("Rejuvenate", Comp, !Comp.HasBuff("Rejuvenate"))) return true;
                        if (MercCastOnGround("Revivification", !Comp.HasBuff("Revivification"), Comp)) return true;
                        if (MercCast("Resurgence", Comp)) return true;
                        if (MercCast("Dark Heal", Comp)) return true;
                        if (MercCast("Dark Infusion", Comp)) return true;
                        if (MercCast("Innervate", Comp, !Comp.HasBuff("Innervate"))) return true;

                        if (MercCast("Rescue", Comp, CHP <= 20)) return true;
                    }
                }
            }
            catch { Logging.Write("Error in CheckCompHealth()"); }

            return false;
        }

        public static bool BGHeals(float minH)
        {
            //float minH = 93f;

            CheckForHeals(minH);
            if (LHUnit == null) return false;

            WaitForCast();

            if (LHUnit.InLineOfSight) MoveTo((TorCharacter)LHUnit, 2.5f);

            if (MercCastOnGround("Kolto Missile", LHH <= minH, LHUnit)) return true;
            if (MercCast("Emergency Scan", LHUnit, LHH <= minH)) return true;
            if (MercCast("Healing Scan", LHUnit, LHH <= minH)) return true;
            if (MercCast("Rapid Scan", LHUnit, LHH <= minH)) return true;
            if (MercCast("Kolto Shield", LHUnit, !LHUnit.HasBuff("Kolto Shield"))) return true;
            if (MyBuffs.Contains("Combat Support Cylinder")) if (MercCast("Rapid Shots", LHUnit, LHH <= minH)) return true;
            return false;
        }

        public static bool CMHeals(Single minH)
        {
            //double minH = 93f;

            CheckForHeals(minH);
            if (LHUnit == null) return false;

            WaitForCast();

            if (LHUnit.InLineOfSight) MoveTo((TorCharacter)LHUnit, 2.5f);

            if (MercCastOnGround("Kolto Bomb", LHH <= minH, LHUnit)) return true;
            if (MercCast("Trauma Probe", LHUnit, !LHUnit.HasBuff("Trauma Probe") && LHH <= minH)) return true;
            if (MercCast("Bacta Infusion", LHUnit, LHH <= minH)) return true;
            if (MercCast("Advanced Medical Probe", LHUnit, LHH <= minH)) return true;
            if (MercCast("Medical Probe", LHUnit, LHH <= minH)) return true;
            if (MyBuffs.Contains("Combat Support Cell")) if (MercCast("Hammer Shot", LHUnit)) return true;

            return false;
        }

        public static bool SeerHeals(float minH)
        {
            //float minH = 93f;

            CheckForHeals(minH);
            if (LHUnit == null) return false;

            WaitForCast();

            if (LHUnit.InLineOfSight) MoveTo((TorCharacter)LHUnit, 2.5f);

            if (MercCast("Force Armor", LHUnit, LHH <= minH && !LHUnit.HasBuff("Force Armor"))) return true;

            if (MercCast("Rejuvenate", LHUnit, LHH <= minH && !LHUnit.HasBuff("Rejuvenate"))) return true;
            if (MercCastOnGround("Salvation", LHH <= minH && !LHUnit.HasBuff("Salvation"), LHUnit)) return true;
            if (MercCast("Benevolence", LHUnit, LHH <= minH)) return true;
            if (MercCast("Deliverance", LHUnit, LHH <= minH)) return true;
            if (MercCast("Healing Trance", LHUnit, LHH <= minH && !LHUnit.HasBuff("Healing Trance"))) return true;
            
            if (MercCast("Rescue", LHUnit, LHH <= 20)) return true;
            
            return false;
        }

        public static bool CorruptionHeals(float minH)
        {
            //float minH = 93f;

            CheckForHeals(minH);
            if (LHUnit == null) return false;

            WaitForCast();

            if (LHUnit.InLineOfSight) MoveTo((TorCharacter)LHUnit, 2.5f);

            if (MercCast("Static Barrier", LHUnit, LHH <= minH && !LHUnit.HasBuff("Static Barrier"))) return true;

            if (MercCastOnGround("Revivification", LHH <= minH && !LHUnit.HasBuff("Revivification"), LHUnit)) return true;
            if (MercCast("Resurgence", LHUnit, LHH <= minH && !LHUnit.HasBuff("Resurgence"))) return true;
            if (MercCast("Dark Heal", LHUnit, LHH <= minH)) return true;
            if (MercCast("Innervate", LHUnit, LHH <= minH && !LHUnit.HasBuff("Innervate"))) return true;
            if (MercCast("Dark Infusion", LHUnit, LHH <= minH)) return true;
            if (MercCast("", LHUnit, LHH <= minH && !LHUnit.HasBuff("Healing Trance"))) return true;

            if (MercCast("Rescue", LHUnit, LHH <= 20)) return true;

            return false;
        }

        public static bool MedicineHeals(float minH)
        {
            //float minH = 93f;

            CheckForHeals(minH);
            if (LHUnit == null) return false;

            WaitForCast();

            if (LHUnit.InLineOfSight) MoveTo((TorCharacter)LHUnit, 2.5f);


            if (MercCast("Kolto Infusion", LHUnit, MyBuffs.Contains("Tactical Advantage") && LHH <= minH)) return true;
            if (MercCast("Surgical Probe", LHUnit, MyBuffs.Contains("Tactical Advantage") && LHH <= minH)) return true;
            if (MercCast("Kolto Probe", LHUnit, LHH <= minH && !LHUnit.HasBuff("Kolto Probe"))) return true;
            if (MercCast("Recuperative Nanotech", LHUnit, LHD <= 1.0F && LHH <= minH)) return true;
            if (MercCast("Kolto Injection", LHUnit, LHH <= minH)) return true;

            return false;
        }

        public static void CheckForDeadComp()
        {
            if (PvP) return;

            if (Me.Level < 10) return;
            if (BuddyTor.Me.IsMounted) return;

            if (Comp!=null) if (!Comp.IsDeleted) if (Comp.IsDead)
            {
                MoveTo(Comp, .50f);
                Thread.Sleep(1500);
                Comp.Interact();
                Thread.Sleep(1500);
                OOC();
            }

        }

        public static bool ValidTarget()
        {
            if (CT != null) if (!IG || TargetInCombat()) if (!CT.IsFriendly && !CT.IsDead && CT.Distance <= 2.8f && !IG) return true;
            return false;
        }

        public static void OOC()
        {
            if (Me.IsDead || Me.HealthPercent < .2f) return;
            //if (Me.CurrentTarget != null) { CorruptionCombatPvP(); return; }

            //if (PvP)
            //{
            //    Thread.Sleep(250);

            //    Inits();

            //    CorruptionHeals(85);
            //    if (Me.CurrentTarget != null) CorruptionCombatPvP();
            //    return;
            //}

            try
            {
                if (Me.InCombat) return;

                if (!PvP)
                {
                    CheckForDeadComp();
                    if (!IG) HarvestIfNec();
                }

                if (Me.IsDead) { Logging.Write("Dead, clearing target and returning."); BuddyTor.Me.ClearTarget(); return; }

                Thread.Sleep(200);
                CheckGC();

                LoadVars();
                Inits();

            } catch {}

            try
            {
                CT = null;
                CT = Me.CurrentTarget;

                //Logging.Write("MySpec: " + MySpec.ToString() + "OpMedicine Spec: " + SkillTreeId.OperativeMedic.ToString());

                if (MySpec == SkillTreeId.MercenaryBodyguard)
                {
                    IsRanged = true;
                    if (!IG) if(!MyBuffs.Contains("High-Velocity Gas Cylinder")) MercCast("High-Velocity Gas Cylinder", BuddyTor.Me);
                    if (IG) if (!MyBuffs.Contains("Combat Support Cylinder")) MercCast("Combat Support Cylinder", BuddyTor.Me);
                    CheckPlayerBuffs("Hunter's Boon");
                    if (IG) BGHeals(95);
                    if (ValidTarget()) { MercCast("Explosive Dart"); BGCombat(); return; }
                }
                else if (MySpec == SkillTreeId.MercenaryArsenal)
                {
                    IsRanged = true;
                    //MercBuffSelf("High-Velocity Gas Cylinder",true);
                    CheckPlayerBuffs("Hunter's Boon");
                    if (ValidTarget()) { MercCast("Explosive Dart"); ArsenalCombat(); return; }
                }
                else if (MySpec == SkillTreeId.MercenaryFirebug)
                {
                    IsRanged = true;
                    if (!MyBuffs.Contains("Combustible Gas Cylinder")) MercCast("Combustible Gas Cylinder", BuddyTor.Me);
                    CheckPlayerBuffs("Hunter's Boon");
                    if (ValidTarget()) { MercCast("Explosive Dart"); FirebugCombat(); return; }
                }
                else if (MySpec == SkillTreeId.CommandoCombatMedic)
                {
                    IsRanged = true;
                    if (IG) CMHeals(95);
                    CheckPlayerBuffs("Fortification");
                    MercBuffSelf("Combat Support Cell", IG);
                    MercBuffSelf("Armor-piercing Cell", !IG);
                    if (ValidTarget()) { CombatMedicCombat(); return; }
                }
                else if (MySpec == SkillTreeId.VanguardShieldSpecialist)
                {
                    IsRanged = true;
                    //if (IG) CMHeals(95);
                    CheckPlayerBuffs("Fortification");
                    MercBuffSelf("Combat Support Cell", IG);
                    MercBuffSelf("Armor-piercing Cell", !IG);
                    if (ValidTarget()) { SSCombat(); return; }
                }
                else if (Me.Class == CharacterClass.Trooper && MySpec == SkillTreeId.None)
                {
                    IsRanged = true;
                    if (IG) CMHeals(95);
                    CheckPlayerBuffs("Fortification");
                    MercBuffSelf("Combat Support Cell", IG);
                    MercBuffSelf("Armor-piercing Cell", !IG);
                    if (ValidTarget()) { CombatMedicCombat(); return; }
                }
                else if (MySpec == SkillTreeId.MarauderAnnihilation)
                {
                    IsRanged = false;
                    WaitForHeals(true, "Channel Hatred", true, 0);
                    CheckPlayerBuffs("Unnatural Might");
                    if (ValidTarget()) { MarauderAnnihilationCombat(); return; }
                }
                else if (MySpec == SkillTreeId.MarauderRage)
                {
                    IsRanged = false;
                    WaitForHeals(true, "Channel Hatred", true, 0);
                    CheckPlayerBuffs("Unnatural Might");
                    if (ValidTarget()) { MarauderRageCombat(); return; }
                }
                else if (MySpec == SkillTreeId.JuggernautImmortal)
                {
                    IsRanged = false;
                    WaitForHeals(true, "Channel Hatred", true, 0);
                    CheckPlayerBuffs("Unnatural Might");
                    if (ValidTarget()) { MarauderAnnihilationCombat(); return; }
                }
                else if (MySpec == SkillTreeId.SentinelWatchman)
                {
                    IsRanged = false;
                    WaitForHeals(true, "Introspection", true, 0);
                    CheckPlayerBuffs("Force Might");
                    if (ValidTarget()) { WatchmanCombat(); return; }
                }
                else if (BuddyTor.Me.Class == CharacterClass.Knight && BuddyTor.Me.AdvancedClass == AdvancedClass.Guardian && (MySpec == SkillTreeId.None || MySpec == SkillTreeId.GuardianDefense))
                {
                    IsRanged = false;
                    WaitForHeals(true, "Introspection", true, 0);
                    CheckPlayerBuffs("Force Might");
                    if (ValidTarget()) { WatchmanCombat(); return; }
                }
                else if (MySpec == SkillTreeId.SageSeer)
                {
                    IsRanged = true;
                    if (IG) SeerHeals(95);
                    WaitForHeals(true, "Meditation", true, BuddyTor.Me.ForceMax * .8f);
                    CheckPlayerBuffs("Force Valor");
                    if (ValidTarget()) { SeerCombat(); return; }
                }
                else if (MySpec == SkillTreeId.SorcererMadness)
                {
                    IsRanged = true;
                    CheckPlayerBuffs("Mark of Power");
                    WaitForHeals(true, "Seethe", true, BuddyTor.Me.ForceMax * .8f);
                    if (ValidTarget()) { MadnessCombat(); return; }
                }
				else if (Me.Class == CharacterClass.Inquisitor)
                {
                    IsRanged = false;
                    CheckPlayerBuffs("Mark of Power");
                    Spell.Cast("Seethe", ret => BuddyTor.Me, ret => BuddyTor.Me.HealthPercent < 95 || BuddyTor.Me.ResourceStat <= 50);
                   
                }
                else if (MySpec == SkillTreeId.SorcererCorruption)
                {
                    IsRanged = true;
                    CheckPlayerBuffs("Mark of Power");
                    WaitForHeals(true, "Seethe", true, BuddyTor.Me.ForceMax * .8f);
                    if (IG || PvP) CorruptionHeals(95);
                    if (ValidTarget()) { CorruptionCombat(); return; }
                }
                else if (MySpec == SkillTreeId.AssassinDarkness)
                {
                    IsRanged = false;
                    CheckPlayerBuffs("Mark of Power");
                    WaitForHeals(true, "Seethe", true, BuddyTor.Me.ForceMax * .6f);
                    if (IG) CorruptionHeals(95);
                    if (ValidTarget()) { DarknessCombat(); return; }
                }
                else if (MySpec == SkillTreeId.ShadowCombat)
                {
                    IsRanged = false;
                    CheckPlayerBuffs("Force Valor");
                    WaitForHeals(true, "Meditation", true, BuddyTor.Me.ForceMax * .8f);
                    MercBuffSelf("Combat Technique", !MyBuffs.Contains("Combat Technique"));
                    if (ValidTarget()) { ConsularKineticCombat(); return; }
                }
                else if (MySpec == SkillTreeId.SniperMarksmanship)
                {
                    IsRanged = true;
                    CheckPlayerBuffs("Coordination");
                    WaitForHeals(true, "Recuperate", true, 80f);
                    MercBuffSelf("Coordination", !MyBuffs.Contains("Coordination"));
                    if (ValidTarget()) { SniperMarksmanshipCombat(); return; }
                }
                else if (MySpec == SkillTreeId.ScoundrelScrapper)
                {
                    IsRanged = true;
                    CheckPlayerBuffs("Lucky Shots");
                    WaitForHeals(true, "Recuperate", true, 80f);
                    MercBuffSelf("Lucky Shots", !MyBuffs.Contains("Lucky Shots"));
                    if (ValidTarget()) { ScrapperCombat(); return; }
                }
                else if (MySpec == SkillTreeId.ScoundrelDirtyFighting)
                {
                    IsRanged = true;
                    CheckPlayerBuffs("Lucky Shots");
                    WaitForHeals(true, "Recuperate", true, 80f);
                    MercBuffSelf("Lucky Shots", !MyBuffs.Contains("Lucky Shots"));
                    if (ValidTarget()) { DirtyFightingCombat(); return; }
                }
                else if (MySpec == SkillTreeId.OperativeMedic)
                {
                    IsRanged = true;
                    CheckPlayerBuffs("Coordination");
                    WaitForHeals(true, "Recuperate", true, 80f);
                    MercBuffSelf("Coordination", !MyBuffs.Contains("Coordination"));
                    if (IG) MedicineHeals(95);
                    if (ValidTarget()) { MedicineCombat(); return; }
                }
                else if (MySpec == SkillTreeId.OperativeLethality)
                {
                    IsRanged = true;
                    CheckPlayerBuffs("Coordination");
                    WaitForHeals(true, "Recuperate", true, 80f);
                    MercBuffSelf("Coordination", !MyBuffs.Contains("Coordination"));
                    if (ValidTarget()) { LethalityCombat(); return; }
                    //if (IG) MedicineHeals(95);
                }
                else if (MySpec == SkillTreeId.OperativeConcealment)
                {
                    IsRanged = false;
                    CheckPlayerBuffs("Coordination");
                    WaitForHeals(true, "Recuperate", true, 80f);
                    MercBuffSelf("Coordination", !MyBuffs.Contains("Coordination"));
                    if (ValidTarget()) { ConcealmentCombat(); return; }
                    //if (IG) MedicineHeals(95);
                }
                else
                {
                    Logging.Write("No matching spec - check the MySpec value (at " + MySpec.ToString() + " against the desired SkillTree value.");
                }

                WaitForComp();

                if (DoRandomGrind && !IG) RunAroundAndKillStuff();
            }
            catch { }
            return;
        }


        public static bool NeedRestRSMin(float minH, float minRS)
        {
            try 
            { if (Comp != null) if ((Comp.Position.X != 0) && Comp.HealthPercent > 0f && Comp.HealthPercent < 85f) return true; }
            catch { }
            
            return (Me.ResourceStat < minRS || Me.HealthPercent < minH);
            //if (NR) Logging.Write("Need Rest...");
        }

        public static bool NeedRestRSMax(float minH, float minRS)
        {
            try
            { if (Comp != null) if (Comp.Position.X != 0 && Comp.HealthPercent < 85f) return true; }
            catch { }

            return (Me.ResourceStat > minRS || Me.HealthPercent < minH);
            //if (NR) Logging.Write("Need Rest...");
        }

        // Where RSIsMinimum=False, we're waiting for a maximum RS, where the RS drops as we rest, like with a Mercenary
        // RSIsMinimum=True, RS goes UP as we rest, as with a Consular etc

        public static void KillStealth()
        {
        //Logging.Write("Entering WaitForHeals...");
            while (MyBuffs.Contains("Stealth"))
            {
                if (!MercCast("Stealth")) MercForcedCast("Force Wave", Me);  // Fucking PITA - Trying to cast Stealth while stealthed - WILL NOT WORK (nor will the Stealth ability)
                Thread.Sleep(1000);
                LoadVars();
            }
        }

        public static void WaitForHeals(bool WaitForComp, string theSpell, bool RSIsMinimum, float RSVal)
        {

            if (Me.InCombat) return;// false;
            if (WaitForComp) if (Comp != null) if (Comp.InCombat) return;// false;

            if (RSIsMinimum)if(!NeedRestRSMin(90, RSVal)) return;// false;
            if (!RSIsMinimum)if(!NeedRestRSMax(90, RSVal)) return;// false;

            if(Me.CurrentTarget==null)KillStealth();

            Buddy.CommonBot.AbilityManager.Cast(theSpell, BuddyTor.Me);

            Thread.Sleep(3500);
            TorCharacter CompX = Comp;

            datLCL = DateTime.Now;
            ForcePulse();

            if (RSIsMinimum) while (DateTime.Now.Subtract(datLCL).TotalSeconds < 10 &&  BuddyTor.Me.IsCasting && NeedRestRSMin(90,RSVal) && !BuddyTor.Me.InCombat) Thread.Sleep(1000);
            if (!RSIsMinimum) while (DateTime.Now.Subtract(datLCL).TotalSeconds < 10 && BuddyTor.Me.IsCasting && NeedRestRSMax(90, RSVal) && !BuddyTor.Me.InCombat) Thread.Sleep(1000);

            Logging.Write("Done with Rest.");
            Buddy.Swtor.Movement.Move(Buddy.Swtor.MovementDirection.Forward, System.TimeSpan.FromMilliseconds(400));
            Thread.Sleep(300);

            StopMoving();
            StopMoving();
            //Logging.Write("Exiting WaitForHeals...");
        }

        public static void ForcePulse()
        {   
            if (DateTime.Now.Subtract(datLFP).TotalSeconds < 5) return;
            try { datLFP = DateTime.Now; Buddy.CommonBot.BotMain.CurrentBot.Pulse(); }
            catch { }
        }

        public static void WaitForComp()
        {
            datLCL = DateTime.Now;
            ForcePulse();

            if (BuddyTor.Me.Companion != null) { while (BuddyTor.Me.Companion.IsCasting && DateTime.Now.Subtract(datLCL).TotalSeconds <= 5) Thread.Sleep(500); }
        }

        //public static float CD(string theSpell)
        //{
        //    float fltCD = 100 ;

        //    //foreach (TorAbility abil in ObjectManager.GetObjects<TorAbility>())
        //        foreach (TorAbility abil in BuddyTor.Me.KnownAbilitiesContainer)
        //            if (abil.Name == theSpell) 
        //        {
        //            //Logging.Write(abil.Name + abil.CooldownTime.ToString() + " " + abil.GlobalCooldownTime.ToString() + " " + abil.CastingTime.ToString() + " " + abil.ChannelingTime.ToString() + " " + abil.IsValid.ToString()); 
        //            fltCD = abil.GlobalCooldownTime; 
        //        }
        //    return fltCD;
        //}
        public static ulong PISkill(string Skill)
        {
                foreach (Buddy.Swtor.Objects.Components.ProfessionInfo pi in BuddyTor.Me.ProfessionInfos)if (pi.Name.Contains(Skill)) return (ulong)pi.CurrentLevel;
                return 0;
        }

        public static bool CanHarvestCorpse(TorNpc unit)
        {
            ulong SkillReq = Math.Max(unit.Level - 3, 0) * 8;
            if (unit != null) if (!unit.IsDeleted) if (unit.IsDead && unit.Distance <= 4.0f &&
                        ((HasBio && unit.CreatureType == Buddy.Swtor.CreatureType.Creature && PISkill("Bioanalysis") >= SkillReq) || (HasScav && unit.CreatureType == Buddy.Swtor.CreatureType.Droid && PISkill("Scavenging") >= SkillReq)) &&
                        StrongOrGreater(unit) && !unit.IsFriendly) if (!Buddy.CommonBot.Blacklist.Contains(unit.Guid)) return true;
            return false;
        }

        public static void MoveToAndInteract(TorNpc unit)
        {
            if (unit.Distance >= 3.0f) MountUp();
            unit.Target();
            //while (unit.Distance > .50f) { Buddy.Swtor.Input.MoveTo(unit.Position, .50f); Thread.Sleep(2000); }
            while (unit.Distance > .50f) { 
                Buddy.Navigation.Navigator.MoveTo(unit.Position); 
                Thread.Sleep(500);
                if (Me.InCombat) { BuddyTor.Me.ClearTarget(); return; }
            }
            StopMoving();
            StopMoving();
            Thread.Sleep(1500);
            if (Me.InCombat) { BuddyTor.Me.ClearTarget(); return; }
            unit.Interact();
            Thread.Sleep(1000);
            while (Me.IsCasting) Thread.Sleep(1000);
            if (Comp!=null) while (Comp.IsCasting) Thread.Sleep(1000);
            if (unit != null) if (CanHarvestCorpse(unit)) { unit.Interact(); Thread.Sleep(3500); return; }
            if (BuddyTor.Me.CurrentTarget != null) MyBlacklist += unit.Guid.ToString() + ";";
            return;

        }

        public static int GetLootableCount()
        {
            int LC=0;

            //var varObj = ObjectManager.GetObjects<TorNpc>();
            //foreach (TorNpc unit in varObj)
            foreach (TorNpc unit in ObjectManager.GetObjects<TorNpc>())
                try
                { if (unit!=null) if (!unit.IsDeleted) if (unit.Distance <= MaxDist && unit.IsDead && unit.IsLootable) LC+=1;
                } catch {}
            Logging.Write("Lootables Count: " + LC.ToString());
            return LC;
        }

        public static bool StrongOrGreater(TorCharacter unit)
        {
            if (unit != null) if (unit.Toughness == CombatToughness.Strong || unit.Toughness == CombatToughness.Boss1 || unit.Toughness == CombatToughness.Boss2 || unit.Toughness == CombatToughness.Player) return true;
            return false;
        }

        public static bool HarvestIfNec()
        {
            if (!HarvestCorpses) return false;
            if (!(HasBio || HasScav)) return false;

            datLCL = DateTime.Now;

            if (DateTime.Now.Subtract(datLC).TotalSeconds < 20) return false;
            //Logging.Write("Entering Harvest...");
            if (BuddyTor.Me.InCombat) return false;
            if (BuddyTor.Me.Companion != null) if (BuddyTor.Me.Companion.InCombat) return false;

            //Trying to use a var (or IEnum) - unreliable
            //var m=ObjectManager.GetObjects<TorNpc>();
            //if (m != null)
            int H = 0;
            //var varObj = ObjectManager.GetObjects<TorNpc>();
            foreach (TorNpc unit in ObjectManager.GetObjects<TorNpc>())
                try
                {

                if (BuddyTor.Me.InCombat) return false;
                if (DateTime.Now.Subtract(datLCL).TotalSeconds >= 2f) return false;
                //if (unit.IsDead)Logging.Write(unit.Name + " " + unit.Toughness.ToString() + " " + unit.CreatureType.ToString()); 
                if (unit != null) if (!unit.IsFriendly && unit.IsDead && unit.Distance <= 5.0f && StrongOrGreater(unit))
                            if (((HasBio && unit.CreatureType == Buddy.Swtor.CreatureType.Creature) || (HasScav && unit.CreatureType == Buddy.Swtor.CreatureType.Droid)))
                                if (!MyBlacklist.Contains(unit.Guid.ToString()))
                                {
                                    Logging.Write("Harvest:  Moving to " + unit.Name + " Distance: " + unit.Distance.ToString() + " Position: " + unit.Position.ToString());
                                    MoveToAndInteract(unit);
                                    H += 1;
                                }
                } 
                catch
                {
                    //Logging.Write("Error in HarvestIfNec.");
                    //return false;
                }

            //varObj = null;

            //Logging.Write("Exiting Harvest - Seconds Elapsed:" + DateTime.Now.Subtract(datLCL).TotalSeconds.ToString());
            datLC = DateTime.Now;
            return (H > 0);
        }

        public static bool CanHarvestObj(TorPlaceable obj)
        {
            ulong LR = obj.Harvestable.ProfessionLevelRequired;
            if (obj.Name.Contains("Security Chest")) return true;
            //Logging.Write("Object Skill Req: " + LR.ToString());
            if (obj != null) if (!obj.IsDeleted) if (obj.Harvestable != null) if (obj.Harvestable.CanHarvest)
                            if (
                                (obj.Harvestable.ProfessionRequired == ProfessionType.Archaeology && HasArch && LR <= PISkill("Archaology")) || (obj.Harvestable.ProfessionRequired == ProfessionType.Bioanalysis && HasBio && LR <= PISkill("Bioanalysis")) ||
                                (obj.Harvestable.ProfessionRequired == ProfessionType.Scavenging && HasScav && LR <= PISkill("Scavenging")) || (obj.Harvestable.ProfessionRequired == ProfessionType.Slicing && HasSlicing && LR <= PISkill("Slicing"))
                               )
                                    return true;
            return false;
        }

        public static bool ScanForPlaceables()
        {
            if (MaxNodesBetweenFights == 0) return false;

            datLCL = DateTime.Now;
            int Harv = 0;
            
            //var objList = ObjectManager.GetObjects<Buddy.Swtor.Objects.TorPlaceable>().OrderBy(t => t.Distance).Where(t => t.Distance <= 5.0f);
            var varObj = ObjectManager.GetObjects<Buddy.Swtor.Objects.TorPlaceable>().OrderBy(t => t.Distance).Where(t => t.Distance <= 12.0f);
            foreach (Buddy.Swtor.Objects.TorPlaceable obj in varObj)
                try
                {
                    //bool CanHarvest = false;
                    //bool CanHarvest = obj.Name.Contains("Chest") && !obj.IsDeleted;
                   

                    //Logging.Write("Obj Found: " + obj.Name + " Distance: " + obj.Distance.ToString("0.0") + " CanHarvest: " + CanHarvest.ToString());
                    if (CanHarvestObj(obj) || obj.Name.Contains("Security Chest"))
                    {
                        Logging.Write("Placeable:  Moving to " + obj.Name + " Distance: " + obj.Distance.ToString() + " Position: " + obj.Position.ToString());
                        MountUp();
                        MountUp();                      // In-Case the first fails, try again

                        {
                            if (obj.Distance >= 3.0f) MountUp();

                            while (obj.Distance > .39f) {
                                if (Me.InCombat) { BuddyTor.Me.ClearTarget(); return false; }
                                Buddy.Navigation.Navigator.MoveTo(obj.Position);
                                //Buddy.Swtor.Input.MoveTo(obj.Position, .50f); Thread.Sleep(2000); 
                            }
                            StopMoving();
                            Thread.Sleep(2000);
                            if (Me.InCombat) { BuddyTor.Me.ClearTarget(); return false; }
                            obj.Interact();
                            Thread.Sleep(4000);

                            if (obj != null) MyBlacklist += obj.Guid.ToString() + ";";

                            ////while (obj != null && obj.Distance > .50f)
                            ////{
                            ////    if (Me.InCombat) return false;
                            ////    //Buddy.Navigation.Navigator.MoveTo(obj.Position);
                            ////    Thread.Sleep(1000);
                            ////}

                            ////Buddy.CommonBot.CommonBehaviors.MoveStop();
                            ////StopMoving();
                            ////Thread.Sleep(2000);

                            ////obj.Interact();
                            ////Thread.Sleep(4500);
                            //////return true;
                            
                            Harv += 1;
                            if (Harv >= MaxNodesBetweenFights) return true;

                        }
                    }
                }
                catch 
                { 
                    //return false; 
                }

            varObj = null;
            Logging.Write("Scan for Nodes: " + DateTime.Now.Subtract(datLCL).TotalSeconds.ToString("#.00") + "s");
            return false;
        }

        public static void MountUp()
        {
            if (MountName != "" && !BuddyTor.Me.IsMounted)
            {
                MercCast(MountName, BuddyTor.Me);
                Thread.Sleep(3000);
            }
        }

        public static bool ScanForLootables()
        {
            //if (KillFest) if (DateTime.Now.Subtract(datLLS).TotalSeconds < 120) return false;

            //foreach (TorNpc unit in ObjectManager.GetObjects<TorNpc>().OrderBy(t => t.Distance))
            int Looted = 0;
            foreach (TorNpc unit in ObjectManager.GetObjects<TorNpc>().OrderBy(t => t.Distance).Where(t => t.Distance <= 10.0f))
                try
                {
                if (unit != null) if (unit.IsDead && unit.IsLootable)
                    {
                        if (Me.InCombat) { BuddyTor.Me.ClearTarget(); return false; }
                        //unit.Target();
                        Logging.Write("Loot:  Moving to " + unit.Name + " Distance: " + unit.Distance.ToString() + " Position: " + unit.Position.ToString());
                        MoveToAndInteract(unit);
                        //Buddy.Navigation.Navigator.MoveTo(unit.Position);
                        Looted += 1;
                        datLLS = DateTime.Now;
                    }
                }
                catch { 
                    //return false; 
                }
            MountUp();
            return Looted > 0;
        }

        public static bool ScanForMobs()
        {
            TorNpc theUnit = null;
            float theDist = MaxDist;

            if (Me.InCombat) return false;
            // Try for LOS Mobs First
            //foreach (TorNpc unit in ObjectManager.GetObjects<TorNpc>().OrderBy(t => t.Distance))
            foreach (TorNpc unit in ObjectManager.GetObjects<TorNpc>().OrderBy(t => t.Distance).Where(t => t.Distance <= MaxDist))
                try
                {
                    if (unit != null) if (!unit.IsDead && unit.Level <= Me.Level + 3 && unit.Distance <= MaxDist && unit.Toughness >= DRGMinToughness && unit.Toughness <= CombatToughness.Boss1 && unit.IsHostile && !unit.IsNeutral && unit.InLineOfSight && !unit.InCombat)
                            if (unit.Distance < theDist || theUnit == null) { theUnit = unit; theDist = unit.Distance; break; }
                }
                catch 
                { 
                    //return false; 
                }
            // Next, Non-LOS Nearby
            ////if (theUnit == null)
            ////    foreach (TorNpc unit in ObjectManager.GetObjects<TorNpc>().OrderBy(t => t.Distance))
            ////        if (unit != null) if (!unit.IsDead && unit.Distance <= MaxDist && unit.Toughness <= CombatToughness.Boss1 && !unit.IsNeutral && unit.IsHostile && !unit.InCombat)
            ////                if (unit.Distance < theDist || theUnit == null) { theUnit = unit; theDist = unit.Distance; break; }

            if (theUnit == null)
                //foreach (TorNpc unit in ObjectManager.GetObjects<TorNpc>().OrderBy(t => t.Distance))
                foreach (TorNpc unit in ObjectManager.GetObjects<TorNpc>().OrderBy(t => t.Distance).Where(t => t.Distance <= MaxDist))
                try
                {
                    if (unit != null) if (!unit.IsDead && unit.Distance <= MaxDist && unit.Toughness>=DRGMinToughness && unit.Toughness <= CombatToughness.Boss1 && unit.IsHostile && !unit.InCombat)
                            if (unit.Distance < theDist || theUnit == null) { theUnit = unit; theDist = unit.Distance; break; }
                }
                catch { return false; }

            // Next, ANYTHING
            if (theUnit == null)
                //foreach (TorNpc unit in ObjectManager.GetObjects<TorNpc>().OrderBy(t => t.Distance))
                foreach (TorNpc unit in ObjectManager.GetObjects<TorNpc>().OrderBy(t => t.Distance).Where(t => t.Distance <= MaxDist))
                    try
                    {
                        if (unit != null) if (!unit.IsDead && unit.Distance <= MaxDist && unit.Toughness <= CombatToughness.Boss1 && !unit.IsFriendly && !unit.InCombat)
                                if (unit.Distance < theDist || theUnit == null) { theUnit = unit; theDist = unit.Distance; break; }
                    }
                    catch { return false; }

            if (theUnit == null) return false;

            if (Me.InCombat) return false;


            Logging.Write("Targeting:  Moving to " + theUnit.Name + " Distance: " + theUnit.Distance.ToString() + " Position: " + theUnit.Position.ToString());
            theUnit.Target();

            if (theUnit.Distance >= 3.0f) MountUp();

            while (theUnit != null && (theUnit.Distance > 2.8f || !theUnit.InLineOfSight))
            {
                if (Me.InCombat) return false;
                Buddy.Navigation.Navigator.MoveTo(theUnit.Position);
                Thread.Sleep(500);
                // If we get hung-up in a fight on the way to the target, abort and rage
                if (BuddyTor.Me.InCombat) { BuddyTor.Me.ClearTarget(); return false; }
            }

            if (theUnit.Distance <= 2.8f && theUnit.InLineOfSight)
            {
                CT = theUnit;
                StopMoving();
                return true;
            }

            StopMoving();

            datLC = DateTime.Now;
            theUnit = null;
            return false;
        }

        //Run around and kill non-friendlies - for use with CombatBot ONLY

        public static bool RunAroundAndKillStuff()
        {
            datLCL = DateTime.Now;

            //Logging.Write("Entering Harvest...");
            if (MeOrCompInCombat()) return false;

            if (DateTime.Now.Subtract(datLCL).TotalSeconds >= 5) return false;

            // Check for Lootables
            //allUnits = Helpers.Units;  // buggy
            //allUnits = ObjectManager.GetObjects<TorNpc>().OrderBy(t => t.Distance).Where(t=>t.Distance<=MaxDist);

            //if (!KillFest || GetLootableCount() >= 10)
            {
                ScanForLootables();
                if (MeOrCompInCombat()) return false;
                if (ScanForNodes) ScanForPlaceables();
                if (MeOrCompInCombat()) return false;
            }
            HarvestIfNec();
            ScanForMobs();

            return false;
        }
    }

    class NoMovement_MovementProvider : IMovementProvider
    {
        public void MoveTowards(Buddy.Common.Math.Vector3 location) { /*empty*/ }
        public void StopMovement() { /*empty*/ }
    };

    class NoMovement_NavigationProvider : INavigationProvider
    {
        public void Clear() { /*empty*/ }
        public MoveResult MoveTo(Buddy.Common.Math.Vector3 destination, string destName, bool useRaycast, bool useStraightLine)
        {
            return (MoveResult.Moved);
        }
    };
}
